-- Main manager file for editor elements.
local t = Def.ActorFrame{}

local EditorLayout = Def.ActorFrame{
	InitCommand=function(self)
		self:xy(SCREEN_WIDTH*.5,SCREEN_BOTTOM-20):zoom(0.8)

		-- Position objects
		local c = self:GetChildren()
		c.Beat:x( -(c.BG:GetZoomedWidth()*.5) + 60 )
		c.LabelBeat:xy( -(c.BG:GetZoomedWidth()*.5) + 60, -20 ):zoom(0.8)
		--c.Second:x( -(c.BG:GetZoomedWidth()*.5) + 180 )
		c.LabelSecond:xy( --[[-(c.BG:GetZoomedWidth()*.5) + 180]] 0, -20 ):zoom(0.8)
		--c.TimingMode:x( (c.BG:GetZoomedWidth()*.5) - 20 )

		c.LabelBPM:xy( (c.BG:GetZoomedWidth()*.5) - 60, -20 ):zoom(0.8)
		c.BPM:x( (c.BG:GetZoomedWidth()*.5) - 60 )

		--c.LabelSnapType:xy( (c.BG:GetZoomedWidth()*.5 - 60), -20 ):zoom(0.8)
		--c.SnapType:x( (c.BG:GetZoomedWidth()*.5 - 60) )
	end,
	EditorStateChangedMessageCommand = function(self,param)
		self:visible( param.EditState ~= "EditState_Playing" )
	end,
	-- This command is called automatically by the editor for
	-- all of the actors available on that screen.
	EditorUpdateCommand=function(self,param)
		local c = self:GetChildren()
		c.Beat:settext( string.format( "%.3f" , param.Beat ) )
		c.Second:settext( string.format( "%.3f" , param.Second ) )
		c.BPM:settext( string.format( "%.3f" , param.BPM ) )
		--if param.SnapType then
		--	c.SnapType:settext( string.format( "%s" , param.SnapType ) )
		--end
	end,
	EditorTimingModeChangedMessageCommand=function(self,param)
		--self:GetChild("TimingMode"):settext( param.Mode )
	end
}

-- Simple Background quad.
EditorLayout[#EditorLayout+1] = Def.Quad{
	Name="BG",
	InitCommand=function(self)
		self:zoomto( SCREEN_WIDTH * .6, 64 ):diffuse(color("#00000088"))
	end
}
-- Obtain beat information element.
EditorLayout[#EditorLayout+1] = Def.BitmapText{ Font = "Common Normal", Text = "Beat", Name = "LabelBeat" }
EditorLayout[#EditorLayout+1] = Def.BitmapText{ Font = "Common Normal", Text = "Second", Name = "LabelSecond" }
EditorLayout[#EditorLayout+1] = Def.BitmapText{ Font = "Common Normal", Text = "BPM", Name = "LabelBPM" }
--EditorLayout[#EditorLayout+1] = Def.BitmapText{ Font = "Common Normal", Text = "SnapType", Name = "LabelSnapType" }
--
EditorLayout[#EditorLayout+1] = Def.BitmapText{ Font = "Common Normal", Name = "Beat" }
EditorLayout[#EditorLayout+1] = Def.BitmapText{ Font = "Common Normal", Name = "Second" }
EditorLayout[#EditorLayout+1] = Def.BitmapText{ Font = "Common Normal", Name = "BPM" }
--EditorLayout[#EditorLayout+1] = Def.BitmapText{ Font = "Common Normal", Name = "SnapType" }
t[#t+1] = EditorLayout


-- Show information for the current chart.
local showInfo = false
local StepsInformation = Def.ActorFrame{
	StepsChangedCommand=function(self)
		self:GetChild("ChartInfo"):playcommand("UpdateStep")
	end,
	EditorStateChangedMessageCommand = function(self,param)
		self:visible( param.EditState ~= "EditState_Playing" )
	end,
	-- Background Quad
	Def.Quad{
		Name="BG",
		InitCommand=function(self)
			self:zoomto(260,200):diffuse(color("#22222277"))
		end
	},

	-- Clickable area to toggle the information.
	LoadModule( "UI/UI.ClickArea.lua" ){
		Debug = true,
		Width = 60,
		Height = 60,
		Action = function(self)
			showInfo = not showInfo
			self:GetParent():stoptweening():easeoutquint(0.2):x( showInfo and 130 or -130 )
		end
	} .. {
		InitCommand=function(self)
			local BG = self:GetParent():GetChild("BG")
			self:xy( BG:GetZoomedWidth()*.5 + 30, -130 )
		end },

	Def.ActorFrame{
		Name="ChartInfo",

		InitCommand=function(self)
			self:RunCommandsRecursively(function(self)
				-- Set a width for the text fields
				if self.settext then
					self:maxwidth(600 * self:GetZoom())
				end
			end)

			local BG = self:GetParent():GetChild("BG")
			self:y( -BG:GetZoomedHeight()*.5 ):playcommand("UpdateStep")
		end,

		UpdateStepCommand=function(self)
			local steps_data = GAMESTATE:GetCurrentSteps(PLAYER_1)
			-- Module from the dance theme that shortens the process of obtaining the difficulty color for the player.
			LoadModule("Color.CustomDiffToColor.lua")(self:GetChild("LevelBG"), steps_data)
			local Diff = steps_data:GetDifficulty()
			self:GetChild("DifficultyName"):settext( THEME:GetString("CustomDifficulty",ToEnumShortString(steps_data:GetDifficulty())) )
			self:GetChild("DifficultyStyle"):settext( THEME:GetString("StepsType",ToEnumShortString(steps_data:GetStepsType())) )
			self:GetChild("DifficultyMeter"):settext( steps_data:GetMeter() )

			self:GetChild("StyleIcon"):Load(
				THEME:GetPathG("", "_StepsVector/"..ToEnumShortString(steps_data:GetStepsType()))
			)
		end,

		Def.Quad{
			Name="LevelBG",
			InitCommand=function(self)
				local BG = self:GetParent():GetParent():GetChild("BG")
				self:zoomto(BG:GetZoomedWidth(),60):valign(1)
				:playcommand("UpdateStep")
			end
		},
	
		Def.BitmapText{
			Name="DifficultyName",
			Font="_plex bold",
			InitCommand=function(self)
				local BG = self:GetParent():GetParent():GetChild("BG")
				self:x(-BG:GetZoomedWidth()*.28):halign(0):uppercase(true):y(-40):zoom(0.5)
			end
		},

		Def.BitmapText{
			Name="DifficultyStyle",
			Font="_plex bold",
			InitCommand=function(self)
				local BG = self:GetParent():GetParent():GetChild("BG")
				self:x(-BG:GetZoomedWidth()*.28):halign(0):uppercase(true):y(-20):zoom(0.4)
			end
		},

		Def.BitmapText{
			Name="DifficultyMeter",
			Font="_plex bold",
			InitCommand=function(self)
				local BG = self:GetParent():GetParent():GetChild("BG")
				self:x(BG:GetZoomedWidth()*.4):halign(1):zoom(0.8):y(-30)
			end
		},
		Def.Sprite {
			Name="StyleIcon",
			InitCommand=function(self)
				self:xy( -self:GetParent():GetParent():GetChild("BG"):GetZoomedWidth()*.38, -30 ):zoom(0.6)
			end
		}
	},

	-- Time for information.
	Def.ActorFrame{
		Name="Labels",
		InitCommand=function(self)
			local indx = 0
			local Background = self:GetParent():GetChild("BG")
			self:RunCommandsRecursively(function(self)
				self:halign(0):zoom(0.7)
				-- Deal with the text strings.
				if self.settext then
					self:settext( THEME:GetString("PaneDisplay",self:GetName()) )
					:xy( -Background:GetZoomedWidth()*.6, -Background:GetZoomedHeight()*.56 + (32 * indx) )
					indx = indx + 1
				end
			end)
		end,
		Def.BitmapText{ Font="_plex bold", Name="Steps" },
		Def.BitmapText{ Font="_plex bold", Name="Jumps" },
		Def.BitmapText{ Font="_plex bold", Name="Hands" },
		Def.BitmapText{ Font="_plex bold", Name="Holds" },
		Def.BitmapText{ Font="_plex bold", Name="Mines" },
		Def.BitmapText{ Font="_plex bold", Name="Rolls" },
		Def.BitmapText{ Font="_plex bold", Name="Lifts" },
		Def.BitmapText{ Font="_plex bold", Name="Fakes" },
	},

	-- The actual values.
	Def.ActorFrame{
		Name="Values",
		InitCommand=function(self)
			local indx = 0
			local Background = self:GetParent():GetChild("BG")
			self:RunCommandsRecursively(function(self)
				self:halign(1):zoom(0.7)
				-- Deal with the text strings.
				if self.settext then
					self:settext( self:GetName() ):y( -Background:GetZoomedHeight()*.56 + (32 * indx) )
					:x( Background:GetZoomedWidth()*.6 )
					indx = indx + 1
				end
			end)
		end,
		EditorUpdateCommand=function(self,params)
			if params.Taps then
				self:GetChild("Taps"):settext( params.Taps )
				self:GetChild("Jumps"):settext( params.Jumps )
				self:GetChild("Hands"):settext( params.Hands )
				self:GetChild("Holds"):settext( params.Holds )
				self:GetChild("Mines"):settext( params.Mines )
				self:GetChild("Rolls"):settext( params.Rolls )
				self:GetChild("Lifts"):settext( params.Lifts )
				self:GetChild("Fakes"):settext( params.Fakes )
			end
		end,
		Def.BitmapText{ Font="_plex bold", Name="Taps" },
		Def.BitmapText{ Font="_plex bold", Name="Jumps" },
		Def.BitmapText{ Font="_plex bold", Name="Hands" },
		Def.BitmapText{ Font="_plex bold", Name="Holds" },
		Def.BitmapText{ Font="_plex bold", Name="Mines" },
		Def.BitmapText{ Font="_plex bold", Name="Rolls" },
		Def.BitmapText{ Font="_plex bold", Name="Lifts" },
		Def.BitmapText{ Font="_plex bold", Name="Fakes" },
	},
}

t[#t+1] = StepsInformation..{
	InitCommand=function(self)
		self:xy( -130, SCREEN_BOTTOM - 200 )
	end
}

local buttons = Def.ActorFrame{
	EditorStateChangedMessageCommand = function(self,param)
		self:visible( param.EditState ~= "EditState_Playing" )
	end
}

buttons[#buttons+1] = LoadModule("UI/UI.GenerateUIWithButtonAction.lua"){
	Width=70,
	Height=40,
	Pos={SCREEN_RIGHT-50, 36},
	Action=function( self )
		SCREENMAN:AddNewScreenToTop("ScreenEditHelp")
	end,
	AddActors=Def.ActorFrame{
		Def.BitmapText{
			Font="_plex bold",
			Text="?",
			OnCommand=function(self)
				self:zoom(0.6):x(-16)
			end
		},
		Def.BitmapText{
			Font="_plex bold",
			Text="F1",
			OnCommand=function(self)
				self:zoom(0.5):x(10)
			end
		}
	}
}

buttons[#buttons+1] = LoadModule("UI/UI.GenerateUIWithButtonAction.lua"){
	Width=160,
	Height=40,
	Pos={SCREEN_RIGHT-170, 36},
	Action=function( self )
		SCREENMAN:AddNewScreenToTop("ScreenEditorOptions")
	end,
	AddActors=Def.ActorFrame{
		Def.Sprite{
			Texture = THEME:GetPathG("", "TitleMenuIcon/Settings"),
			OnCommand=function(self)
				self:zoom(0.08):x(-54)
			end
		},
		Def.BitmapText{
			Font="_plex bold",
			Text="SETTINGS",
			OnCommand=function(self)
				self:zoom(0.4):x(-20):halign(0)
			end
		}
	}
}

t[#t+1] = buttons

t[#t+1] = LoadActor("./ChartInfo.lua")

return t
