local t = Def.ActorFrame{}

-- Background layer that represents the overlaying object.
local BottomA = Def.ActorFrame{
	BGHeight = 42,
	InitCommand=function(self) self:y(SCREEN_BOTTOM) end,
}

BottomA[#BottomA+1] = Def.Quad {
	InitCommand=function(self)
		self:zoomto( SCREEN_WIDTH, BottomA.BGHeight ):diffuse( color("#0E0F20") ):halign(0):valign(1)
	end
}

-- Upper border of overlaying object
BottomA[#BottomA+1] = Def.Quad {
	InitCommand=function(self)
		self:zoomto( SCREEN_WIDTH, 4 ):y(-BottomA.BGHeight):diffuse( color("#025F55") ):halign(0):valign(1)
	end
}

-- t[#t+1] = BottomA

local takenscreenshot = false
local calcwidth = 300

local ContinueButton = {
    Width = calcwidth*.75,
    Height = 36,
    Pos = { SCREEN_CENTER_X , SCREEN_BOTTOM - 22},
    Action = function( self )
		SCREENMAN:PlayStartSound()
        SCREENMAN:GetTopScreen():StartTransitioningScreen( "SM_GoToNextScreen", 0 )
    end,
    AddActors = Def.BitmapText{
		Font = "Common Normal", Text = "&START; Continue",
		InitCommand=function(self)
			self:zoom(0.8)
		end
	}
}

-- Screenshot Button
local screenshot = {
    Width = calcwidth*.5,
    Height = 36,
    Pos = { SCREEN_CENTER_X + 210 , SCREEN_BOTTOM - 22},
    Action = function(self)
		local pref = "OutFox/"
		local success, path = SaveScreenshot( nil , false, false, pref)

		if success then
			self:GetParent():finishtweening():diffuse( color("#777777") )
			:easeoutquint(0.5):diffuse( color("#ffffff") )
		end
    end,
    AddActors = Def.BitmapText{
		Font = "Common Normal", Text = "&SELECT; Screenshot",
		InitCommand=function(self)
			self:zoom(0.8)
		end
	}
}

local isMenuOpen = false
local OpenMenu = {
    Width = calcwidth*.5,
    Height = 36,
    Pos = { SCREEN_CENTER_X - 210 , SCREEN_BOTTOM - 22},
    Action = function( self )
		isMenuOpen = not isMenuOpen
        MESSAGEMAN:Broadcast( (isMenuOpen and "Show" or "Close") .. "PopupMenu")
    end,
    AddActors = Def.BitmapText{ Font = "Common Normal", Text = "More Options" }
}

local staticButtonAC = Def.ActorFrame{
	OffCommand=function(self)
		self:stoptweening():easeinexpo(0.25):diffusealpha(0):addy(20)
	end
}

local buttonAC = Def.ActorFrame{
	InitCommand=function(self)
		self:diffusealpha(0)
	end,
	ShowPopupMenuMessageCommand=function(self)
		isMenuOpen = true
		self:linear(0.2):diffusealpha(1)
	end,
	ClosePopupMenuMessageCommand=function(self)
		self:linear(0.2):diffusealpha(0)
	end,
	OffCommand=function(self)
		self:stoptweening():easeinexpo(0.25):diffusealpha(0):addy(20)
	end
}

local btnGen = LoadModule("UI/UI.GenerateUIWithButtonAction.lua")

staticButtonAC[#staticButtonAC+1] = btnGen( ContinueButton )
staticButtonAC[#staticButtonAC+1] = btnGen( OpenMenu )
staticButtonAC[#staticButtonAC+1] = btnGen( screenshot )

local allowedToRetry = GAMESTATE:GetCoinMode() == "CoinMode_Home" and GAMESTATE:IsEventMode()

local buttonsList = {
	{ "Continue", "ScreenProfileSave" },
	{ "Restart", function(self)
		GAMESTATE:GetCurrentSong():PlayPreviewMusic()
		SCREENMAN:GetTopScreen():SetNextScreenName("ScreenPlayerOptionsLua"):StartTransitioningScreen("SM_GoToNextScreen")
	end, allowedToRetry },
	{ "Return To Main Menu", function(self)
		GAMESTATE:Env()["ForceEndSession"] = true
		SCREENMAN:GetTopScreen():SetNextScreenName("ScreenProfileSave"):StartTransitioningScreen("SM_GoToNextScreen")
	end },
	{ "Cancel", function(self)
		isMenuOpen = false
		MESSAGEMAN:Broadcast("ClosePopupMenu")
		-- Do nothing
	end }
}

local baseYPosButtons = -(50 * (#buttonsList/2) - 25)
for i,v in ipairs(buttonsList) do
	buttonAC[#buttonAC+1] = btnGen{
		Width = 260,
		Height = 36,
		Pos = { SCREEN_CENTER_X, SCREEN_CENTER_Y + baseYPosButtons + (50 * (i-1))},
		Action = function( self )
			if v[3] ~= nil and not v[3] then return end

			if type(v[2]) == "function" then
				v[2](self)
			else
				SCREENMAN:GetTopScreen():SetNextScreenName(v[2]):StartTransitioningScreen("SM_GoToNextScreen")
			end
		end,
		AddActors = Def.BitmapText{ Font = "Common Normal", Text = v[1] }
	}
end

t[#t+1] = staticButtonAC
t[#t+1] = buttonAC

return t