local Player = ...
local CurPage = 1
local CurZoom = 1
local CurPositionZoom = 0
local PageWithBreakdown = 0
local inPopUpMenu = false
local onlineTest = NETMAN and NETMAN:IsConnectionEstablished()

local Pages = {
    "TimingDistribution",
	"ColumnScore",
    "FitnessInfo",
}

local colorLight = ColorLightTone(PlayerCompColor(Player))
local colorDark = ColorDarkTone(PlayerCompColor(Player))
local playerColorComp = PlayerCompColor(Player)

local PVHandler

if onlineTest then
	table.insert(Pages, 2, "ScoreComp")
end

for k,v in pairs(Pages) do
    if v == "TimingDistribution" then
        PageWithBreakdown = k
    end
end

-- TODO: Merge this control listener to a singular one to avoid duplicates.
local t = Def.ActorFrame{
	InitCommand=function(self)
		PVHandler = self
	end,
    -- Attach the controler so the player can access the information.
    OnCommand=function(self)
        SCREENMAN:GetTopScreen():AddInputCallback( LoadModule("Lua.InputSystem.lua")(self) )
        for k,v in pairs( Pages ) do
            self:GetChild("Page"..k):diffusealpha( 0 )
        end
        self:playcommand("UpdateViewPort",{Current = CurPage})
    end,

    -- These will change the page the player is currently looking.
    MenuLeftCommand=function(self) self:playcommand("MoveChoice", {-1}) end,
    MenuRightCommand=function(self) self:playcommand("MoveChoice", {1}) end,

	SelectCommand=function(self)
		inPopUpMenu = true
		MESSAGEMAN:Broadcast("ShowPopupMenu")
	end,

	ShowPopupMenuMessageCommand=function(self)
		inPopUpMenu = true
	end,
	ClosePopupMenuMessageCommand=function(self)
		inPopUpMenu = false
	end,

    MenuUpCommand=function(self) self:playcommand("PerformZoom", {1}) end,
    MenuDownCommand=function(self) self:playcommand("PerformZoom", {-1}) end,

	ViewingDataFromScoreMessageCommand=function(self)
		CurPage = 1
		self:playcommand("MoveChoice", {0})
	end,

    MoveChoiceCommand=function(self,param)
		if inPopUpMenu then return end

        if CurZoom ~= 1 and CurPage == PageWithBreakdown then
			local newpos = CurPositionZoom + param[1]
			local limitarea = (CurZoom*CurZoom)
			if newpos > limitarea then newpos = limitarea end
			if newpos < -limitarea then newpos = -limitarea end
			CurPositionZoom = newpos
            self:GetChild("Page"..PageWithBreakdown):playcommand("ScatterMove",{ XPos = newpos, Zoom = CurZoom })
            return
        end

        CurPage = CurPage + param[1]

        if CurPage > #Pages then CurPage = 1 end
        if CurPage < 1 then CurPage = #Pages end
    
        self:playcommand("UpdateViewPort",{Current = CurPage})
    end,

    PerformZoomCommand=function(self,param)
		if inPopUpMenu then return end
        if CurPage ~= PageWithBreakdown then return end
        CurZoom = CurZoom + param[1]

        if CurZoom > 10 then CurZoom = 10 end
        if CurZoom < 1 then CurZoom = 1 end

        self:GetChild("Page"..PageWithBreakdown):playcommand("ScatterZoom",{ XPos = CurPositionZoom, Zoom = CurZoom })

		if CurZoom == 1 then
			CurPositionZoom = 0
			self:GetChild("Page"..PageWithBreakdown):playcommand("ScatterMove",{ XPos = 0, Zoom = 1 })
			return
		end

		local newpos = CurPositionZoom
		local limitarea = (CurZoom*CurZoom)
		if newpos > limitarea then newpos = limitarea end
		if newpos < -limitarea then newpos = -limitarea end
		CurPositionZoom = newpos
        self:GetChild("Page"..PageWithBreakdown):playcommand("ScatterMove",{ XPos = newpos, Zoom = CurZoom })
    end,

    UpdateViewPortCommand=function(self)
        for k,v in pairs( Pages ) do
            self:GetChild("Page"..k):stoptweening():linear(0.1):diffusealpha( 0 )
            if CurPage == k then
                self:GetChild("Page"..k):diffusealpha(1):zoom(1)
                --self:GetChild("HeaderPage"):playcommand("UpdateText",{ NewText = v })
            end
        end
		self:GetChild("InfoTab"):playcommand("UpdateInfo",{Text=THEME:GetString("ScreenEvaluation",Pages[CurPage])})
    end
}

--Background elements
local calcwidth = 440
local playerdataxpos = Player == PLAYER_1 and 228 or -228
local MBG
local arguments = {}

t[#t+1] = Def.Sprite{
	Texture=THEME:GetPathG("","Evaluation/PlayerArea"),
	InitCommand=function(self)
		self:diffuse(PlayerColor(Player)):xy( SCREEN_CENTER_X-playerdataxpos, SCREEN_CENTER_Y+70 ):zoom(1.3)
		self:rotationy( Player == PLAYER_1 and 0 or 180 )
		MBG = self
	end,
	OffCommand=function(self)
		self:stoptweening():easeinexpo(0.25):diffusealpha(0)
	end
}

t[#t+1] = LoadModule("Wheel/Objects/PlayerArea.lua"){
	Player = Player,
	Width = 380,
	}..{
	InitCommand=function(self)
		self:xy( SCREEN_CENTER_X-(playerdataxpos), SCREEN_CENTER_Y-( MBG:GetZoomedHeight()*.5 ) + 2 )
	end,
	OffCommand=function(self)
		self:stoptweening():easeinexpo(0.25):diffusealpha(0)
	end
}

local JName = LoadModule("Gameplay/Timing.GetNames.lua").Names

local OffsetTable = STATSMAN:GetCurStageStats():GetPlayerStageStats( Player ):GetOffsetData()
local sm = LoadModule("Gameplay.Median.lua")( Player, OffsetTable, {300,230} )

local arguments = {
	Player = Player,
	-- Will be updated on the fly.
	Width = 440,
	Height = 500,
	CurTiming = LoadModule("Options.ReturnCurrentTiming.lua")(),
	TapNoteNames = JName,
	Offsets = OffsetTable,
	Median = sm,
	-- visual settings
	fade_out_speed = 0.2,
	fade_out_pause = 0.1,
	off_wait = 0.75,
	isDetailed = true
}

-- Difficulty Container
t[#t+1] = LoadActor( "PlayerSpecific/Objects/DiffContainer.lua", {
	Player = Player,
} )..{
	OnCommand=function(self)
		self:xy( SCREEN_CENTER_X-playerdataxpos, SCREEN_CENTER_Y-188 )
	end
}

-- Load the pages in.
-- In the advanced mode, the regular stats will always be shown.
t[#t+1] = LoadActor( "PlayerSpecific/Statistics.lua", arguments ).. {
    OnCommand=function(self)
        self:xy( SCREEN_CENTER_X-playerdataxpos, SCREEN_CENTER_Y + 50 )
		:playcommand("UpdateWidth",{Width=MBG:GetZoomedWidth()})
    end,
	OffCommand=function(self)
		self:stoptweening():easeinexpo(0.25):diffusealpha(0)
	end
}

local ogMusicRate = GAMESTATE:GetSongOptionsObject("ModsLevel_Preferred"):MusicRate()
t[#t+1] = Def.BitmapText{
	Font="Common Normal",
	Text=string.format("%.2fx Music Rate", ogMusicRate),
	InitCommand=function(self)
		self:xy( SCREEN_CENTER_X-playerdataxpos, SCREEN_CENTER_Y+190 ):zoom(0.8)
		:visible( ogMusicRate ~= 1 )
	end,
	OffCommand=function(self)
		self:linear(0.2):diffusealpha(0)
	end,
	ViewingDataFromScoreMessageCommand=function(self,params)
		if not params.rate then return end
		self:settext( params.rate )
	end,
	ResetToPlayerScoreMessageCommand=function(self)
		self:settext( GAMESTATE:GetPlayerState(Player):GetPlayerOptionsString("ModsLevel_Preferred") )
	end
}

t[#t+1] = Def.BitmapText{
	Font="Common Normal",
	Text=GAMESTATE:GetPlayerState(Player):GetPlayerOptionsString("ModsLevel_Preferred"),
	InitCommand=function(self)
		self:xy( SCREEN_CENTER_X-(playerdataxpos + calcwidth*.5 - 44), SCREEN_CENTER_Y+260 ):zoom(0.8)
		:align(0,1):diffuse( colorLight )
		:wrapwidthpixels( 440 ):vertspacing(-4)
	end,
	OffCommand=function(self)
		self:linear(0.2):diffusealpha(0)
	end,
	ViewingDataFromScoreMessageCommand=function(self,params)
		if not params.modifiers then return end
		self:settext( params.modifiers )
	end,
	ResetToPlayerScoreMessageCommand=function(self)
		self:settext( GAMESTATE:GetPlayerState(Player):GetPlayerOptionsString("ModsLevel_Preferred") )
	end
}

t[#t+1] = LoadActor("PlayerSpecific/TimingInfo.lua",{Player = Player, Width = calcwidth})..{
	InitCommand=function(self)
		self:xy( SCREEN_CENTER_X-playerdataxpos, SCREEN_CENTER_Y+292 ):zoom(0.8)
	end,
	OffCommand=function(self)
		self:linear(0.2):diffusealpha(0)
	end
}

-- Force values for the extended version of the info pane.

arguments.IsStaticElement = false
arguments.Height = 590
local spacingX = Player == PLAYER_2 and -34 or 34
-- arguments.Height = 500

t[#t+1] = Def.Sprite{
	Texture=THEME:GetPathG("","Evaluation/InfoArea"),
	OnCommand=function(self)
		self:diffuse(PlayerColor(Player)):xy( SCREEN_CENTER_X+playerdataxpos - spacingX, SCREEN_CENTER_Y + 24 ):zoom(1.3)
	end,
	OffCommand=function(self)
		self:stoptweening():easeinexpo(0.25):diffusealpha(0)
	end
}

-- Describe which page we're in.
t[#t+1] = LoadActor( "PlayerSpecific/Objects/PageView.lua", {
	PaneColor = ColorLightTone(PlayerColor(Player)),
	BGColor = ColorDarkTone(PlayerColor(Player)),
	Width = 370,
} )..{
	OnCommand=function(self)
		self:xy(SCREEN_CENTER_X+playerdataxpos-spacingX, SCREEN_CENTER_Y-arguments.Height*.5 + 64 )
	end,
}

-- These will be controlled by the InputFilter.
for k,v in pairs( Pages ) do
    t[#t+1] = LoadActor( "PlayerSpecific/"..v , arguments )..{
        Name = "Page"..k,
        InitCommand=function(self)
            self:xy( SCREEN_CENTER_X+playerdataxpos - spacingX, SCREEN_CENTER_Y + 24 )
			:playcommand("UpdateWidth",{Width=MBG:GetZoomedWidth()})
        end
    }
end

t[#t+1] = LoadActor("PageCount.lua", {numPages = #Pages, Player = Player, Width = arguments.Width, Current = CurPage})..{
	InitCommand=function(self)
		self:xy( SCREEN_CENTER_X+playerdataxpos - spacingX, SCREEN_CENTER_Y - arguments.Height*.5 + 90 )
	end,
	OffCommand=function(self)
		self:linear(0.2):diffusealpha(0)
	end,
}

-- Only perform this if the user has a valid token to attempt.
if NETMAN and NETMAN:IsConnectionEstablished() and PROFILEMAN:GetProfile(Player):IsOnlineRegistered() then
	local hasSentScore = false
	local cornerSavePos = Player == PLAYER_1 and 170 or (SCREEN_WIDTH - 170)
	t[#t+1] = Def.ActorFrame{
		InitCommand=function(self)
			self:xy( cornerSavePos, SCREEN_BOTTOM - 24 )
			:zoom(0.8)

			self:GetChild("Image"):diffuse( PlayerColor(Player) )
			:zoom(0.6)
			self:GetChild("Status"):settext( "Submitting..." )
			self:GetChild("loadicon"):x( -self:GetChild("Status"):GetZoomedWidth() * .5 - 20 )
		end,
		OffCommand=function(self)
			self:linear(0.2):diffusealpha(0)
		end,
		
		ResponseScoreSaveMessageCommand=function(self,params)
			local res = params.Response
			local achieved = res and res["metadata"]["status"] == "success" or false
			self:GetChild("loadicon"):stoptweening():linear(0.1):diffusealpha(0)
			self:GetChild("Status"):settext( THEME:GetString("ScreenEvaluation","ScoreSubmit".. (achieved and "Success" or "Failure")) )
			if not achieved and res and res["metadata"]["message"] == "Score Already Submitted" then
				self:GetChild("Status"):settext( THEME:GetString("ScreenEvaluation","ScoreSubmitExisting") )
			end
			self:GetChild("Image"):Load( THEME:GetPathG("OnlineStatus", achieved and "online" or "offline") )
			:x( -self:GetChild("Status"):GetZoomedWidth()*.5 - 30)
		end,
		OnlineScoreSaveFailedMessageCommand=function(self)
			self:GetChild("Status"):settext( THEME:GetString("ScreenEvaluation","ScoreSubmitFailure") )
			self:GetChild("Image"):Load( THEME:GetPathG("OnlineStatus","offline") ):x( -self:GetChild("Status"):GetZoomedWidth()*.5-30 )
			self:GetChild("loadicon"):stoptweening():linear(0.1):diffusealpha(0)
		end,
		Def.BitmapText{ Name="Status", Font = "Common Normal" },
		Def.Sprite{ Name="Image" },
		Def.Sprite{
			Name="loadicon",
			Texture=THEME:GetPathG("Loading","icon"),
			InitCommand=function(self)
				self:spin():zoom(0.1):effectmagnitude(0,0,400)
			end,
			UpdateScoresCommand=function(self)
				self:stoptweening():linear(0.1):diffusealpha(0)
			end
		}
	}
end

return t
