local t = Def.ActorFrame{}

local Args = ...
local p = Args.Player
local Width = Args.Width
local Height = Args.Height

local StgStats = STATSMAN:GetCurStageStats():GetPlayerStageStats(p)
local PlProfile = PROFILEMAN:GetProfile(p)
-- The number is the machine profile.
local IsProfileMachine = PlProfile:GetDisplayName() == ""

local Information = {
	{ "CaloriesBurnedSong", StgStats:GetCaloriesBurned() },
	{ "CaloriesToday", PlProfile:GetCaloriesBurnedToday() },
	{ "CaloriesTotal", PlProfile:GetTotalCaloriesBurned() }
}

------------------------------------------
-- Items
------------------------------------------
for ind,val in ipairs( Information ) do
	t[#t+1] = Def.ActorFrame{
		InitCommand=function(self)
			self:xy(-Width*.475 + ((Width*.3 + 10) * (ind-1)),-Height*.5 + 100)
		end,
		OffCommand=function(self) self:linear(0.2):diffusealpha(0) end,
		Def.BitmapText {
			Font = "_Bold",
			Text=ToUpper( Screen.String(val[1]) ),
			InitCommand=function(self)
				self:zoom(0.6):maxwidth(200):horizalign(left):y(-15)
				if GAMESTATE:GetNumPlayersEnabled() > 1 then
				end
			end
		},
		Def.BitmapText{
			Font = "_Plex Bold Large",
			InitCommand=function(self)
				self:zoom(0.75):maxwidth(260):horizalign(left):y( 10 )
				:diffuse(ColorLightTone(PlayerColor(p))):diffusetopedge(ColorLightTone(PlayerCompColor(p)))
			end,
			Text=math.floor( val[2] )
		}
	}
end

local Yspace_FitnessGoal = -Height*.5 + 156
------------------------------------------
-- Machine Profile Detector
------------------------------------------
if IsProfileMachine then
	t[#t+1] = Def.BitmapText {
		Font = "_Bold",
		InitCommand=function(self)
			self:zoom(0.64):y(Yspace_FitnessGoal):wrapwidthpixels(Width + 100)
			:diffusealpha(0.8)
		end,
		OffCommand=function(self) self:linear(0.2):diffusealpha(0) end,
		Text=Screen.String("NoProfileSet")
	}
end


------------------------------------------
-- Goal Marker
-- Note: Goals should not appear here if there's not a profile.
------------------------------------------
local GoalMarking = Def.ActorFrame{
	OffCommand=function(self) self:linear(0.2):diffusealpha(0) end,
	InitCommand=function(self)
		local c = self:GetChildren()
		-- Verify if we need to show any kind of goal before processing data.
		-- 0 = Time
		-- 1 = Calories
		-- 2 = None
		if PlProfile:GetGoalType() < 2 then
			-- Did the player achieve the goal?
			if GAMESTATE:IsGoalComplete( p ) then
				c.CurrentGoalLabel:settext( THEME:GetString(Var "LoadingScreen","GoalAchieved") ):halign(0.5):xy(0, Yspace_FitnessGoal)
				c.GoalToGoLabel:visible(false)
				c.CurrentGoalValue:visible(false)
				c.CurrentGoalProgress:visible(false)
				return
			end
			
			-- We need to check if the current goal we have is either calories or time, so process this information.
			local stats = STATSMAN:GetCurStageStats()
			if PlProfile:GetGoalCalories() ~= 0 then
				self.GoalAmmount = string.format( "%.2f cal", PlProfile:GetGoalCalories() - stats:GetPlayerStageStats(p):GetCaloriesBurned() )
			else
				self.GoalAmmount = SecondsToMMSS(PlProfile:GetGoalSeconds() - stats:GetGameplaySeconds())
			end

			-- Now send the information to the respective actor.
			c.CurrentGoalValue:settext( self.GoalAmmount )
			c.CurrentGoalProgress:GetChild("Percentage"):settext( string.format(THEME:GetString(Var "LoadingScreen", "ProgressComplete"), FormatPercentScore(GAMESTATE:GetGoalPercentComplete(p))) )
		else
			-- No goal, replace placement of the labels to indicate such event.
			c.CurrentGoalLabel:settext( Screen.String("NoGoal") ):halign(0.5):xy(0, Yspace_FitnessGoal)
			:wrapwidthpixels(600):diffusebottomedge(ColorLightTone(PlayerCompColor(p))):diffusealpha(0.5)

			-- And hide these since we only need to show one BitmapText actor.
			c.GoalToGoLabel:visible(false)
			c.CurrentGoalValue:visible(false)
			if c.CurrentGoalProgress then
				c.CurrentGoalProgress:visible(false)
			end
		end
	end
}

GoalMarking[#GoalMarking+1] = Def.BitmapText{
	Font = "_Bold",
	Name = "CurrentGoalLabel",
	Text = Screen.String("CurrentGoal"),
	InitCommand = function(self)
		self:xy(-Width*.5 + 8,Yspace_FitnessGoal - 20):halign(0)
		:diffuse(Color.White):diffusebottomedge(ColorLightTone(PlayerCompColor(p)))
	end
}

GoalMarking[#GoalMarking+1] = Def.BitmapText{
	Font = "_Medium",
	Name = "GoalToGoLabel",
	Text = Screen.String("GoalToGo"),
	InitCommand = function(self) self:zoom(1):xy(-60,Yspace_FitnessGoal + 15):halign(0) end
}

GoalMarking[#GoalMarking+1] = Def.BitmapText{
	Font = "_Bold",
	Name = "CurrentGoalValue",
	InitCommand = function(self) self:zoom(1.2):xy(-80,Yspace_FitnessGoal + 15):halign(1) end
}

GoalMarking[#GoalMarking+1] = Def.ActorFrame{
	Name = "CurrentGoalProgress",
	Condition = PlProfile:GetGoalType() < 2,
	InitCommand=function(self)
		self:xy(Width*.25,Yspace_FitnessGoal + 5)
	end,
	Def.BitmapText{
		Font = "_Medium",
		Name = "Percentage",
		InitCommand = function(self) self:zoom(.8):xy(Width*.25 - 8,-10):halign(1) end
	},
	Def.Quad{
		InitCommand=function(self)
			self:zoomto( Width*.5 - 10, 14 ):xy( 0, 10 ):diffuse(Color.Black)
		end
	},
	Def.Quad{
		InitCommand=function(self)
			self:zoomto( Width*.5 - 12, 12 ):xy( 0, 10 ):diffuse(PlayerColor(p))
			:cropright( 1 -GAMESTATE:GetGoalPercentComplete(p) )
		end
	}
}

if not IsProfileMachine then
	t[#t+1] = GoalMarking
end

local stageoffset = 0
local numitems = Args.StatisticsRemainsStill and 6 or 9
local itemheight = 42

local HistoryPane = Def.ActorFrame{
	InitCommand=function(self)
		self:playcommand("ChangeOffset")
	end,
	OffCommand=function(self) self:linear(0.2):diffusealpha(0) end,
	ChangeOffsetCommand=function(self)
		local st = GAMESTATE:GetCurrentStyle():GetStepsType()
		for i = 1, numitems do
			if STATSMAN:GetPlayedStageStats(i+stageoffset) then
				local stagestats = STATSMAN:GetPlayedStageStats(i+stageoffset)
				local playerstats = stagestats:GetPlayerStageStats( p )
				local song = stagestats:GetPlayedSongs()[1]
				local steps = playerstats:GetPlayedSteps()[1]

				-- If no steps are given, that means the player did not play that chart.
				if steps then
					self:GetChild("Item"..i):GetChild("SongTitle"):settext( song:GetDisplayMainTitle() )
					self:GetChild("Item"..i):GetChild("SongArtist"):settext( string.sub(song:GetDisplayArtist(), 0, 50) )
					self:GetChild("Item"..i):GetChild("CalBurned"):settext(
						--string.format( "%.2f Cal", playerstats:GetCaloriesBurned())
						FormatPercentScore	(playerstats:GetPercentDancePoints())
					)

					local courseType = GAMESTATE:IsCourseMode() and SongOrCourse:GetCourseType() or nil;
					local cd = GetCustomDifficulty(st, steps:GetDifficulty(), courseType);
					self:GetChild("Item"..i):GetChild("Diff"):settext(
						THEME:GetString("CustomDifficulty",ToEnumShortString(steps:GetDifficulty())) .. "  " .. string.format("%.4g",steps:GetMeter())
					) -- :diffuse(ColorLightTone(ColorLightTone(CustomDifficultyToColor(cd))));
					
					if song:GetBannerPath() then
						self:GetChild("Item"..i):GetChild("AFT"):GetChild("SongBanner"):Load( song:GetBannerPath() ):scaletocover(0,0,Width,itemheight)
					end
				end
			end
		end
	end
}


local BannerAreaWidth = Width
for i = 1, numitems do
	HistoryPane[#HistoryPane+1] = Def.ActorFrame{
		Name="Item"..i,
		OnCommand=function(self) self:y( -Height*.5 + 164 + ( itemheight * i ) ) end,

		Def.Quad{
			OnCommand=function(self)
				self:zoomto( BannerAreaWidth - 8, itemheight-2 ):diffuse( ColorDarkTone( PlayerColor(p) ) )
			end
		},

		Def.ActorFrameTexture{
			Name="AFT",
			InitCommand=function(self)
				self:SetWidth(BannerAreaWidth - 8):SetHeight(itemheight-2):Create()
			end,
			Def.Sprite { Name="SongBanner", InitCommand=function(self) self:xy( BannerAreaWidth*.5, itemheight*.5 ) end }
		},
		
		Def.Sprite{
			OnCommand=function(self)
				self:SetTexture( self:GetParent():GetChild("AFT"):GetTexture() )
				self:diffusealpha(0.25):fadeleft(0.5):faderight(0.5)
			end
		},

		Def.BitmapText{
			Font="Common Normal", Name="SongTitle",
			InitCommand=function(self)
				self:halign(0):xy( -BannerAreaWidth*.5 + 8, -6 ):zoom(0.8):shadowlength(2)
			end
		},

		Def.BitmapText{
			Font="Common Normal", Name="SongArtist",
			InitCommand=function(self)
				self:halign(0):xy( -BannerAreaWidth*.5 + 8, 8 ):zoom(0.6):shadowlength(2)
			end
		},

		Def.BitmapText{
			Font="Common Normal", Name="CalBurned",
			InitCommand=function(self)
				self:halign(1):xy( BannerAreaWidth*.5 - 8, 8 ):zoom(0.7):shadowlength(2)
			end
		},

		Def.BitmapText{
			Font="Common Normal", Name="Diff",
			InitCommand=function(self)
				self:halign(1):xy( BannerAreaWidth*.5 - 8, -6 ):zoom(0.7):shadowlength(2)
			end
		}
	}
end

t[#t+1] = HistoryPane

return t
