-- Density graph information
local Arguments = ...

local pn = Arguments.Player
local SctMargin = Arguments.Width
local maxheight = Arguments.Height
local needsTransparency = Arguments.Transparent
local npsdata = Arguments.NoteData
local Conf = Arguments.Conf

local lastlife = 0

local t = Def.ActorFrame{}
local reverse = Enum.Reverse(TapNoteScore)

local sum = 0
local needsFullGraph = Conf:Get("showfullgraph",false)

local function UpdateAFrame( self, delta )
    sum = sum + delta
    if sum > 0.3 then
        self:playcommand("UpdateLifeGraph")
        sum = 0
    end
    
    if not self.NeedsMove then return end
    if needsFullGraph then return end

    local cursecond = GAMESTATE:GetCurMusicSeconds()
    local length = scale( cursecond, self.Start, self.time, 0, 1 )
    local graphStartPoint = (SctMargin*.5 * self.sctconv)
    self:x( scale( length, 0, 1, SctMargin*.5 + graphStartPoint, SctMargin*.5 - graphStartPoint ) )

    -- The number 10 is a magic trick here.
    local scmargScale = scale( length, 0, 1, -SctMargin*.5, SctMargin*.5 ) - ((SctMargin*.5) / self.sctconv )
    -- If the song is long, then we must figure out a way to stop drawing ALL of the vertices going through the scatterplot,
    -- because it's going to lag exponentially based on the number of notes.
    if self:GetChild("ScatterGraphNormal").verts[1] then
        local dotX = self:GetChild("ScatterGraphNormal").verts[1][1][1]
        if scmargScale > dotX then
            table.remove( self:GetChild("ScatterGraphNormal").verts, 1 )
        end
    end
    
    -- As this is a line, we have to remove two items, but the loop will take care of that.
    if self:GetChild("ScatterGraphMiss").verts[1] then
        local dotX = self:GetChild("ScatterGraphMiss").verts[1][1][1]
        if scmargScale > dotX then
            table.remove( self:GetChild("ScatterGraphMiss").verts, 1 )
        end
    end

    -- Same thing for lifebars.
    if self:GetChild("Graphd").verts[1] then
        local life = self:GetChild("Graphd").verts[1][1]
        local xpos = life[1]
        if scmargScale > xpos then
            -- Special case if the next y point is the same as the current one.
            if self:GetChild("Graphd").verts[2][1][2] == life[2] then
                -- Reset the X position of this vertice to the border.
                self:GetChild("Graphd").verts[1][1][1] = scmargScale
                if scmargScale > self:GetChild("Graphd").verts[2][1][1] then
                    table.remove( self:GetChild("Graphd").verts, 1 )
                end
            else
                table.remove( self:GetChild("Graphd").verts, 1 )
            end
        end
    end
end

local amv = Def.ActorFrameTexture{
    InitCommand=function(self)
        self:SetTextureName("DensityGraph"..pn):EnableAlphaBuffer(true):SetWidth( SctMargin ):SetHeight( maxheight ):Create()
    end,

    Def.Quad{
		Condition = not needsTransparency,
        OnCommand=function(self)
            self:zoomto( SctMargin, maxheight ):align(0,0):diffuse(Color.Black):diffusealpha(0.75)
        end
    },
    
    Def.ActorFrame{
        InitCommand=function(self)
            self:xy( SctMargin*.5, maxheight*.5 )
            self.NeedsMove = false
			if GAMESTATE:GetCurrentSong() then
				self.Start = math.min(GAMESTATE:GetCurrentSong():GetTimingData():GetElapsedTimeFromBeat(0), 0)
			end
            self.sctconv = 1
        end,
        OnCommand=function(self)
            self:playcommand("UpdateCurrentInformation"):playcommand("CheckTime")

            self:SetUpdateFunction(UpdateAFrame)
        end,
        -- We're done, stop using the function.
        OffCommand=function(self) self:SetUpdateFunction(nil) end,
        CheckTimeCommand=function(self)
            self:finishtweening():zoomtowidth( 1 ):x( SctMargin*.5 )
			self.NeedsMove = self.time > 150 or Conf:Get("alwaysmovegraph",false)
			
            if self.NeedsMove and not needsFullGraph then
                self:zoomtowidth( self.sctconv )
            end
        end,
        CurrentSongChangedMessageCommand=function(self) self:finishtweening():playcommand("UpdateCurrentInformation") end,
        UpdateCurrentInformationCommand=function(self)
            self:finishtweening()
			self.Start = math.min(GAMESTATE:GetCurrentSong():GetTimingData():GetElapsedTimeFromBeat(0), 0)
            self.time = GAMESTATE:GetCurrentSong():GetLastSecond()
            self.sscale = self.time > 150 and self.time > 500 and 280 or scale( self.time, 150, 500, 0, 280 ) or 0
            self.sctconv = SctMargin / (300 - self.sscale )
            
            local names = { "Graph","Graphd","ScatterGraphNormal","ScatterGraphMiss" }
            
            for v in ivalues(names) do self:GetChild(v):playcommand("UpdateInfo") end
            
			self:playcommand("CheckTime")
        end,

        Def.ActorMultiVertex{
            Name="Graph",
            OnCommand=function(self)
                self.verts = {}
                self:SetDrawState{Mode="DrawMode_QuadStrip"}
            end,
            UpdateInfoCommand=function(self)
                local stats = STATSMAN:GetCurStageStats()
                if not stats then return end

				if GAMESTATE:IsCourseMode() then
					if not GAMESTATE:GetCourseSongIndex() then return end
				end
                
				local song = GAMESTATE:GetCurrentSong()

				if GAMESTATE:IsCourseMode() then
					song = GAMESTATE:GetCurrentTrail(pn):GetTrailEntries()[ GAMESTATE:GetCourseSongIndex()+1 ]:GetSong()
				end
                if song then
                    local data = npsdata:GetCurrentData()
					local steps = GAMESTATE:GetCurrentSteps(pn)
					if GAMESTATE:IsCourseMode() then
						steps = GAMESTATE:GetCurrentTrail(pn):GetTrailEntries()[ GAMESTATE:GetCourseSongIndex()+1 ]:GetSteps()
					end
                    if data and steps then
                        local tnp, npst = data.PeakNPS,data.Density
                        local SongMargin = {
                            Start = 0,
                            End = song:GetLastSecond(),
                        }
                        -- Grab every instance of the NPS data.
                        self.verts = {}
                        if npst then
                            for k,v in pairs( npst ) do
                                -- Each NPS area is per MEASURE. not beat. So multiply the area by 4 beats.
                                local t = steps:GetTimingData():GetElapsedTimeFromBeat((k-1)*4)
                                -- With this conversion on t, we now apply it to the x coordinate.
                                local x = scale( t, SongMargin.Start, SongMargin.End, -SctMargin*.5, SctMargin*.5 )
                                -- Now scale that position on v to the y coordinate.
                                local y = math.round( v > tnp and (-maxheight*.5 + 2) or scale( v, 0, tnp, maxheight*.5, -maxheight*.5 + 2 ) )
                                -- And send them to the table to be rendered.
                                if x <= SctMargin then
                                    if #self.verts > 2 and self.verts[#self.verts][1][2] == y and self.verts[#self.verts-2][1][2] == y then
                                        self.verts[#self.verts][1][1] = x
                                        self.verts[#self.verts-1][1][1] = x
                                    else
                                        self.verts[#self.verts+1] = {{x, maxheight/2, 0}, PlayerColor(pn) }
                                        self.verts[#self.verts+1] = {{x, y, 0}, ColorLightTone(PlayerColor(pn))}
                                    end
                                end
                            end
                            self:SetNumVertices( #self.verts ):SetVertices( self.verts )
                        end
                    end
                end
            end
        },

		Def.SongMeterDisplay{
			Name="CompleteQuad",
			StreamWidth=SctMargin,
			Condition=Conf:Get("progressquad",true),
			StartFromZero=true,
			Stream=Def.Quad{
				InitCommand=function(self)
					self:zoomtoheight(maxheight):diffuse( color("#00000099") )
				end
			},
			Tip=Def.ActorFrame{}
		},

        Def.ActorMultiVertex{
            Name="Graphd",
            OnCommand=function(self)
                self.verts = {}
                self:SetDrawState{Mode="DrawMode_LineStripM"}:SetLineWidth(3)
            end,
            UpdateInfoCommand=function(self)
                self.verts = {}
                self:SetNumVertices( #self.verts ):SetVertices( self.verts )
                self:finishtweening()

                if GAMESTATE:GetCurrentSong() then
                    self.SStart = 0
                    self.SEnd = GAMESTATE:GetCurrentSong():GetLastSecond()
                end
            end,
            UpdateLifeGraphCommand=function(self)
                local stats = STATSMAN:GetCurStageStats()
                if not stats then return end

                local mpStats = stats:GetPlayerStageStats( pn )
                local v = mpStats:GetCurrentLife()
                
                if v then
                    local t = GAMESTATE:GetSongPosition():GetMusicSeconds()

                    if t > self.SEnd then return end
                    -- With this conversion on t, we now apply it to the x coordinate.
                    local x = scale( t, self.SStart, self.SEnd, -SctMargin/2, SctMargin/2 )
                    -- Now scale that position on v to the y coordinate.
                    local y = scale( v, 0, 1, maxheight/2, -maxheight/2 + 1 )
                    
                    local setc = Color.White

                    if Conf:Get("colorlifescatter",true) then
                        local worst = mpStats:GetWorstTapNoteScore()
                        if reverse[worst] > reverse["TapNoteScore_W4"] then
                            setc = Brightness(
                                JudgmentLineToColor("JudgmentLine_".. ToEnumShortString( worst ))
                            , 0.8)
                        end
                    end

                    if t > 0 then
                        if #self.verts > 2 then
                            if (lastlife ~= v or setc ~= self.verts[#self.verts][1][3]) then
                                lastlife = v
                                self.verts[#self.verts+1] = {{x, y, 0}, setc }
                            else
                                self.verts[#self.verts][1][1] = x
                            end
                        else
                            self.verts[#self.verts+1] = {{x, y, 0}, setc }
                        end
                        self:SetNumVertices( #self.verts ):SetVertices( self.verts )
                    end
                end
            end,
            OffCommand=function(self) self:sleep(0.15):decelerate(0.3):addy(-75) end
        },

        Def.ActorMultiVertex{
            Name="ScatterGraphNormal",
            OnCommand=function(self)
                self.verts = {}
                self:SetDrawState{Mode="DrawMode_Points"}:SetPointState(true):SetPointSize(2)

                self.LastNoteMargin = 0
            end,
            UpdateInfoCommand=function(self)
                self.verts = {}
                self:SetNumVertices( #self.verts ):SetVertices( self.verts )
                self:finishtweening()

                if GAMESTATE:GetCurrentSong() then
                    self.SStart = 0
                    self.SEnd = GAMESTATE:GetCurrentSong():GetLastSecond()
                end
            end,
            JudgmentMessageCommand=function(self,params)
                self:finishtweening()
                if not Conf:Get("judgmentscatter",false) then return end
                -- When we recieve a judgment, we must filter the player to avoid null data.
                if params.Player == pn and params.Notes then
                    -- Let´s check what column was just hit
                    -- to prevent bugs, we remove instances of Pump's checkpoints.
                    if params.TapNoteOffset and (params.TapNoteScore ~= "TapNoteScore_CheckpointHit" and params.TapNoteScore ~= "TapNoteScore_CheckpointMiss") then
                        -- Time will be our total offset for the current vertices
                        local time = GAMESTATE:GetCurMusicSeconds() - params.TapNoteOffset
                        
                        -- We need to define the type of judgment that it is.
                        local noff = params.TapNoteScore == "TapNoteScore_Miss" and false or params.TapNoteOffset
                        
                        -- Now that we have time, we have to place it on the scatter.
                        -- The spacing is from the start to the end of the song's visible area, not the entire song.
                        -- This is to prevent offsync margins.

                        local x = scale( time, self.SStart, self.SEnd, -SctMargin/2, SctMargin/2 )
                        
                        -- Now we differenciate each judgment by giving it color.
                        local val = "JudgmentLine_"..ToEnumShortString( params.TapNoteScore )
                        local ColorType = JudgmentLineToColor( val )
                        
                        -- Now create a Square, with the judgment color
                        if noff then
                            local y = scale( noff, 0.25, -0.25, maxheight/2, -maxheight/2 )
                            table.insert( self.verts, {{x, y, 0}, ColorType } )
                            self:SetVertices( self.verts )
                        end
                    end
                end
            end
        },

        Def.ActorMultiVertex{
            Name="ScatterGraphMiss",
            OnCommand=function(self)
                self.verts = {}
                self:SetDrawState{Mode="DrawMode_Lines"}:SetLineWidth(2)
            end,
            UpdateInfoCommand=function(self)
                self.verts = {}
                self:SetNumVertices( #self.verts ):SetVertices( self.verts )
                self:finishtweening()

                if GAMESTATE:GetCurrentSong() then
                    self.SStart = math.min(GAMESTATE:GetCurrentSong():GetTimingData():GetElapsedTimeFromBeat(0), 0)
                    self.SEnd = GAMESTATE:GetCurrentSong():GetLastSecond()
                end
            end,
            JudgmentMessageCommand=function(self,params)
                self:finishtweening()
                if not Conf:Get("judgmentscatter",false) then return end
                -- When we recieve a judgment, we must filter the player to avoid null data.
                if params.Player == pn and params.Notes then
                    if params.HoldNoteScore then return end
                    if params.TapNoteScore ~= "TapNoteScore_Miss" then return end

                    -- Time will be our total offset for the current vertices
                    local time = GAMESTATE:GetCurMusicSeconds()
                        
                    -- Now that we have time, we have to place it on the scatter.
                    -- The spacing is from the start to the end of the song's visible area, not the entire song.
                    -- This is to prevent offsync margins.

                    local x = scale( time, self.SStart, self.SEnd, -SctMargin/2, SctMargin/2 )
                    local ColorType = Alpha( JudgmentLineToColor( "JudgmentLine_Miss" ), 0.6 )
                    
                    table.insert( self.verts, {{x, maxheight/2, 0}, ColorType} )
                    table.insert( self.verts, {{x, -maxheight/2, 0}, ColorType} )
                    
                    self:SetVertices( self.verts )
                end
            end
        }
    }
}

-- Now with everything created, it's time to send it over.
return Def.ActorFrame{
    CurrentSongChangedMessageCommand=function(self) self:playcommand("UpdateInfo") end,
    UpdateInfoCommand=function(self)
        if Conf:Get("labelnps",false) then
            if npsdata:GetCurrentData() then
                self:GetChild("NPSData"):settext(
					string.format( THEME:GetString("ScreenGameplay","MaxNPS"), npsdata:GetCurrentData().PeakNPS ) .. " / " ..
					string.format( THEME:GetString("ScreenGameplay","AvgNPS"), npsdata:GetCurrentData().AverageNPS )
				)
            end
        end
    end,

    -- The ActorMultiVertex in question, which is inside an ActorFrameTexture,
    amv,

    -- Now we proyect said AFT into a sprite, so we can see it.
    Def.Sprite{ Name="Graph", Texture="DensityGraph"..pn },

    -- And as an extra thing, let's indicate the current NPS too.
    Def.BitmapText{ Name="NPSData", Font="Common Normal", InitCommand=function(self) self:uppercase(true):diffuse(ColorLightTone(PlayerColor(pn))) end,
        OnCommand=function(self) self:xy( SctMargin/2 - 10, -maxheight/2 - 20 ):align(1,0):zoom(0.8):shadowlengthy(2) end
    }
}