local Args = ...
local needsStats = Args.requireStats
local pn = Args.Player
local NoteData = Args.NoteData
local t = Def.ActorFrame{}
local aspectRatio = GetScreenAspectRatio()

local needsCentering = (Center1Player() or GAMESTATE:GetIsFieldCentered(pn) or aspectRatio <= 0.66)

local playerPosition = THEME:GetMetric("ScreenGameplay","PlayerP".. string.sub(pn,-1) .."OnePlayerOneSideX")
local score_x_position = pn == PLAYER_1 and 45 or -45
local diff_x_position = pn == PLAYER_1 and -95 or 95
local sml = clamp(scale( aspectRatio, 1.3, 1.7, 50, 0 ), 0, 50) * ( pn == PLAYER_2 and -1 or 1 )
local ScaleItems = aspectRatio < 1 and scale( aspectRatio, 0.5625, 0.75, 1, 1.3 ) or 1.4

if needsCentering then
	playerPosition = SCREEN_CENTER_X
	sml = 0

	local pos = {
		["StyleType_OnePlayerOneSide"] = 80,
		["StyleType_OnePlayerTwoSides"] = 110,
	}

	local position = pos[GAMESTATE:GetCurrentStyle():GetStyleType()] or 0
	playerPosition = SCREEN_CENTER_X + (pn == PLAYER_1 and -position or position)
	if needsStats then
		playerPosition = SCREEN_CENTER_X
	end
end

local pAC = Def.ActorFrame{
	OnCommand=function(self)
		self:x( playerPosition - sml ):y( 50 ):zoom(ScaleItems)
		if aspectRatio < 1 then
			self:x( SCREEN_WIDTH*.35 )
		end

		-- is the player in reverse? Move it.
		if GAMESTATE:GetPlayerState(pn):GetCurrentPlayerOptions():Reverse() > 0.5 then
			self:y( SCREEN_BOTTOM - 50 )
		end
	end
}

local profile = PROFILEMAN:GetProfile(pn)
local inFitnessMode = profile:GetGoalType() ~= 2
if inFitnessMode then
	if profile:GetGoalCalories() == 0 and profile:GetGoalSeconds() == 0 then
		inFitnessMode = false
	end
end
-- SCREENMAN:SystemMessage( tostring(PROFILEMAN:GetProfile(pn):GetGoalType()) )

-- Load the profile elements, like the avatar and difficulty number.
pAC[#pAC+1] = loadfile( THEME:GetPathB("ScreenGameplay","underlay/layered/profileArea.lua") )(pn)..{ InitCommand=function(self) self:x(diff_x_position) end }
-- Load the score elements, like the percentage and density graph.
pAC[#pAC+1] = loadfile( THEME:GetPathB("ScreenGameplay","underlay/layered/".. (inFitnessMode and "fitness" or "score") .."Area.lua") )({Player = pn, NoteData = NoteData})..{
	InitCommand=function(self) self:x(score_x_position) end
}

t[#t+1] = Def.ActorFrame{
	InitCommand=function(self)
		local notefieldWidth = GAMESTATE:GetStyleFieldSize(pn)
		self:xy( playerPosition - 130, SCREEN_BOTTOM - 40 )
		:diffusealpha(0.8)

		-- if needsCentering then
		-- 	self:x(SCREEN_CENTER_X)
		-- end
	end,
	OnCommand=function(self)
		self:sleep(3):easeoutexpo(0.5):diffusealpha(0)
	end,

	Def.BitmapText{
		Font="_Bold",
		Name="CurrentSpeed",
		InitCommand=function(self)
			self:halign(0):zoom(1.25)
			local speed,value = LoadModule("Gameplay/Speed.Calculate.lua")(pn)
			self:settext( value )
		end,
	},

	NOTESKIN:LoadActorForNoteSkin( "Left", "Tap Note", GAMESTATE:GetPlayerState(pn):GetPlayerOptions("ModsLevel_Preferred"):NoteSkin() )..{
		OnCommand=function(self) self:x(-30):zoom(0.8) end
	},
}

t[#t+1] = pAC

return t
