-- Scoreboard functions
local usePercentScore = PREFSMAN:GetPreference("PercentageScoring")
local Args = ...
local NoteData = Args.NoteData
local pn = Args.Player
local PlayerSavePath = CheckIfUserOrMachineProfile(string.sub(pn,-1)-1).."/OutFoxPrefs.ini"
local usesGraphOnTop = LoadModule("Config.Load.lua")("OverTopGraph",PlayerSavePath)
local showEX = true

return Def.ActorFrame{
	Def.Sprite{ Name="Base", Texture=THEME:GetPathG("","Gameplay/ScoreArea"),
		InitCommand=function(self)
			self:diffuse( PlayerColor(pn) ):rotationy( pn == PLAYER_2 and 180 or 0 )
		end
	},

	-- Passively drawn ActorMultiVertex to cover the out of shape area of the player graph.
	Def.ActorMultiVertex{
		Condition=usesGraphOnTop,
		OnCommand=function(self)
			-- Construct map of vertices first.
			local verts = {
				{ {0, 10, 0}, Color.White },
				{ {10, 0, 0}, Color.White },
				{ {10, 10, 0}, Color.White }
			}

			-- Set the drawmode to triangles.
			self:SetDrawState({Mode="DrawMode_Triangles"})
			-- Now send the vertice data created previously.
			:SetVertices(verts)

			-- Position the mask to the corner to mask
			local side = (pn == PLAYER_2 and -1 or 1)
			local offset = 10
			self:xy( (self:GetParent():GetChild("Base"):GetZoomedWidth()*.5)*side - (offset*side), 12 )
			:rotationy( pn == PLAYER_2 and 180 or 0 )

			:MaskSource()
		end
	},
	
	LoadActor( "../PlayerSpecific/PlayerGraph.lua", { Player = pn, Width= 147, Height = 41, NoteData = NoteData, UseMask = true } )..{
		Condition=usesGraphOnTop,
		OnCommand=function(self) self:xy( 0.2 , 0 ) end
	},

	-- Score display
	Def.PercentageDisplay{
		AutoRefresh=true,
		Condition = GAMESTATE:GetPlayMode() ~= "PlayMode_Endless",
		Percent=Def.BitmapText{
			Font="_Plex Numbers 40px",
			InitCommand=function(self)
				self:strokecolor(Color.Black)
				:diffuse(ColorLightTone(PlayerColor(pn))):diffusebottomedge(PlayerColor(pn)):maxwidth(248/0.75)
				:halign(1)
			end,
		},
		InitCommand=function(self)
			self:x( self:GetParent():GetChild("Base"):GetZoomedWidth()*.5 - 10 ):zoom(0.6)
			:LoadFromStats( GAMESTATE:GetPlayerState(pn), STATSMAN:GetCurStageStats():GetPlayerStageStats(pn) )
		end,
	},
	Def.BitmapText {
		Condition = GAMESTATE:GetPlayMode() == "PlayMode_Endless",
		Font="_Plex Numbers 40px",
		InitCommand=function(self)
			self:xy(score_x_position + (usesGraphOnTop and 120 or 0), (usesGraphOnTop and 16 or 0) ):zoom( usesGraphOnTop and 0.6 or 0.75)
			:diffuse(ColorLightTone(PlayerCompColor(pn))):diffusebottomedge(ColorLightTone(PlayerColor(pn))):maxwidth(248/0.75)
			:settext(usePercentScore and " 0.00%" or 0)
			:halign( usesGraphOnTop and 1 or 0.5 )
			if usesGraphOnTop then
				self:diffuse( Color.White ):strokecolor(Color.Black)
			end
			self:queuecommand("UpdateTimer")
		end,
		UpdateTimerCommand=function(self)
			local vStats = STATSMAN:GetCurStageStats():GetPlayerStageStats( pn )
			self:finishtweening():settext( SecondsToMMSSMsMs( vStats:GetAliveSeconds() ) )
			:sleep(1/60):queuecommand("UpdateTimer")
		end,
		OffCommand=function(self) self:sleep(0.15):decelerate(0.3):addy(-75) end
	},
}
