-- Fun fact, the engine won't load profiles by the time this screen has started, it's when the Continue command is issued.
return Def.ActorFrame{
	OnCommand=function(self)
		if not NETMAN or not NETMAN:IsConnectionEstablished() then SCREENMAN:GetTopScreen():Continue() return end
		GAMESTATE:LoadProfiles()

		local succeded = true
		for _,pn in ipairs( PlayerNumber ) do
			if GAMESTATE:IsPlayerEnabled(pn) then
				-- local profile = PROFILEMAN:GetProfile(pn)
				if PROFILEMAN:IsPersistentProfile(pn) then
					-- Message will be broadcasted to the system layer.
					local status = NETMAN:ClientLogin(pn)
					if not status then
						MESSAGEMAN:Broadcast("SystemNotification",{
							Text= PROFILEMAN:GetProfile(pn):GetDisplayName() .. " has no assigned token to login to OutFox Online.",
							Icon = THEME:GetPathG("OnlineStatus","offline")
						})
						self:GetChild("Loading"):queuecommand("Hide")
						-- Ask the user if they want to assign a token.
						self:playcommand("RequestSkip")
						-- succeded = false
					end
				else
					succeded = false
				end
			end
		end
		if not succeded then
			SCREENMAN:GetTopScreen():Continue()
		end
	end,
	CancelSkipMessageCommand=function(self)
		-- The user has requested to not assign a token. Let them through.
		SCREENMAN:GetTopScreen():Continue()
	end,
	ResponseClientLoginMessageCommand=function(self,params)
		local status = type(params.Response) ~= "nil"
		local text = "Could not connect"
		if status then
			-- lua.ReportScriptError(rin_inspect(params.Response))
			if params.Response["metadata"] then
				for _,pn in ipairs( PlayerNumber ) do
					if GAMESTATE:IsPlayerEnabled(pn) then
						text = "Logged " .. PROFILEMAN:GetProfile(pn):GetDisplayName()
						NETMAN:ProfileLoad(pn)
					end
					-- SCREENMAN:GetTopScreen():Continue()
				end
			end
		end

		MESSAGEMAN:Broadcast("SystemNotification",{
			Text= text .. " to OutFox Online.",
			Icon = THEME:GetPathG("OnlineStatus",status and "online" or "offline")
		})
	end,
	ResponseProfileLoadMessageCommand=function(self,params)
		-- Now in here, we get a response to load up modifiers.
		local status = type(params.Response) ~= "nil"
		-- lua.ReportScriptError(rin_inspect(params.Response))
		if status and params.Response["metadata"] then
			for _,pn in ipairs( PlayerNumber ) do
				if PROFILEMAN:GetProfile(pn):GetGUID() == params.Response["metadata"]["profile_uuid"] then
					-- This is the profile!
					if type(params.Response["profile"]["options"]) == "string" then
						GAMESTATE:GetPlayerState(pn):GetPlayerOptions("ModsLevel_Preferred"):FromString( params.Response["profile"]["options"] )
					end
				end
			end
		end

		SCREENMAN:GetTopScreen():Continue()
	end,

	Def.ActorFrame{
		Name="Loading",
		InitCommand=function(self)
			self:xy( SCREEN_CENTER_X, SCREEN_BOTTOM - 140 )
		end,
		OnCommand=function(self)
			self:diffusealpha(0):linear(0.2):diffusealpha(1)
		end,
		HideCommand=function(self)
			self:stoptweening():diffusealpha(0)
		end,
		Def.Sprite{
			Name="loadicon",
			Texture=THEME:GetPathG("Loading","icon"),
			InitCommand=function(self)
				self:spin():effectmagnitude(0,0,400):zoom(0.3)
			end
		},

		Def.BitmapText{
			Font="_Medium",
			Text="Logging in...",
			InitCommand=function(self)
				self:y(-70)
			end
		}
	},
	LoadActor("InitialSetup/OnlineSkipWindow.lua")
}