local zoomScale = LoadModule("Lua.Resize.lua")
GAMESTATE:Reset()
for pn in ivalues(PlayerNumber) do
    GAMESTATE:UnjoinPlayer(pn)
end

-- Useful function for determine a range and limiting it without overflowing
local function LimitRange(Val, Start, Endv, Min, Max)
    -- Unfortunately, the scale function available does not properly limit the value
    local EndVal = scale(Val, Start, Endv, Min, Max)
    -- Here we do the work for it
    if EndVal > Max then return Max end
    if EndVal < Min then return Min end
    -- If the value was scaled without exceeding limits, then return it
    return EndVal
end

-- Adapted from above, this will allow us to properly scale items horizontally on the screen
local function WidthScale(Min, Max)
    local EndValue = scale(SCREEN_WIDTH, 960, 1280, Min, Max)
    if EndValue > Max then return Max end
    if EndValue < Min then return Min end
    return EndValue
end

-- List of menu options available
local MenuList = {
    {
        {"Play Now", "ScreenProfileLoad"},
        {"Profiles", "ScreenOptionsManageProfiles"},
        {"Fitness", "ScreenSelectProfileFitness"},
    },
    {
        {"Settings", "OptionsMenu"},
        {"Editor", "ScreenEditMenu"},
        {"Quit", "ScreenExit"},
    }
}

-- Dynamically set the button size from screen size
local QuadSize = {WidthScale(230, 300), 200}
-- Limit the screen margins to avoid buttons displaying out of screen
local MarginArea = { SCREEN_CENTER_X - 300, SCREEN_CENTER_X + 300}
-- Vertical position of the buttons
local ButtonAreaY = SCREEN_CENTER_Y - 20
-- Check if the mascot option is enabled
local MascotEnabled = LoadModule("Config.Load.lua")("ShowMascotCharacter","Save/OutFoxPrefs.ini")

-- Here we will first display the logo and the mascot if required
-- The mascot must be done before the main menu so that the buttons can overlap it
local t = Def.ActorFrame{
	Def.Sprite{
		Texture=THEME:GetPathG("ScreenTitleMenu", "logo/_text"),
		InitCommand=function(self)
			self:xy(SCREEN_CENTER_X - 460, SCREEN_CENTER_Y - 190)
            :zoom(LimitRange(SCREEN_WIDTH, 960, 1280, 0.4, 0.5))
			:halign(0)

            LoadModule("Tools.Discord.lua"):CustomScreenState("Title Menus")
		end,
		OffCommand=function(self)
			self:easeinexpo(0.25):addx(-100):diffusealpha(0)
		end
	},
    Def.Sprite{
		Condition=MascotEnabled,
		Texture=THEME:GetPathG("ScreenTitleMenu", "logo/_mascot"),
		InitCommand=function(self)
            if MascotEnabled then
                self:xy(SCREEN_WIDTH * .85, SCREEN_HEIGHT + 200)
                :zoom(1.2):diffusealpha(0.5):valign(1)
            end
        end,
        OffCommand=function(self)
			self:easeinexpo(0.25):addx(100):diffusealpha(0)
		end
	}
}

-- Our table of current gamemodes for the drop down menu selector
local exlist = {
	{ Name = "dance", Icon = THEME:GetPathG("","_StepsVector/Dance_Single") },
    { Name = "pump", Icon = THEME:GetPathG("","_StepsVector/Pump_Single") },
    { Name = "smx", Icon = THEME:GetPathG("","_StepsVector/Smx_Single") },
    { Name = "techno", Icon = THEME:GetPathG("","_StepsVector/Techno_Single8") },
    { Name = "be-mu", Icon = THEME:GetPathG("","_StepsVector/Bm_Single7") },
    { Name = "po-mu", Icon = THEME:GetPathG("","_StepsVector/Pnm_Nine.svg") },
    { Name = "gddm", Icon = THEME:GetPathG("","_StepsVector/Gddm_New") },
    { Name = "gdgf", Icon = THEME:GetPathG("","_StepsVector/Gdgf_Five") },
	{ Name = "gh", Icon = THEME:GetPathG("","_StepsVector/Gh_Solo") },
    { Name = "taiko", Icon = THEME:GetPathG("","_StepsVector/Taiko") },
    { Name = "bongo", Icon = THEME:GetPathG("","_StepsVector/Bongo") },
    { Name = "para", Icon = THEME:GetPathG("","_StepsVector/Para_Single") },
    { Name = "kbx", Icon = THEME:GetPathG("","_StepsVector/Kb9_Single") },
    { Name = "ez2", Icon = THEME:GetPathG("","_StepsVector/Ez2_Single.svg") },
    { Name = "ds3ddx", Icon = THEME:GetPathG("","_StepsVector/Ds3ddx_Single5.svg") },
    { Name = "maniax", Icon = THEME:GetPathG("","_StepsVector/Maniax_Single.svg") },
	{ Name = "stepstage", Icon = THEME:GetPathG("","_StepsVector/Stepstage_Twin") },
    { Name = "kickbox", Icon = THEME:GetPathG("","_StepsVector/Kickbox_Human") },
}

-- Determine a margin for the items on the table
for i=1,#exlist do
	if type(exlist[i]) == "table" then
		exlist[i].Margin = 28
	end
end

local dropdown = LoadModule("UI/UI.DropDown.lua"){
    Width = 200,
    Height = 48,
    List = exlist,
    currentItem = function(self, list)
        local curgame = GAMESTATE:GetCurrentGame():GetName()
        for k,v in pairs( list ) do
            if v.Name == curgame then
                return k
            end
        end
        return 1
    end,
    perItemAction = function(self, list, a)
        GAMEMAN:SetGame( (list[a].Name and list[a].Name or list[a]), THEME:GetCurThemeName() )
    end,
    UseMetatable = true,
}

-- Everything related to the main menu buttons will be done from this module
t[#t+1] = Def.ActorFrame{
    LoadModule("UI/UI.MainMenu.lua")(MenuList, QuadSize, MarginArea, ButtonAreaY, dropdown)
}

-- Draw the amount of songs, groups and courses on the top right corner
t[#t+1] = Def.ActorFrame{
    InitCommand=function(self)
        self:xy(SCREEN_WIDTH * .75, 40)
    end,
    OnCommand=function(self)
		self:diffusealpha(0):easeoutexpo(0.2):diffusealpha(1)
	end,
	OffCommand=function(self)
		self:easeinexpo(0.25):addy(-100):diffusealpha(0)
	end,

    LoadModule("UI/UI.ButtonBox.lua")(300, 48),

    Def.Text{
        Name="NumSongs",
        Font=THEME:GetPathF("","IBMPlexSans-Medium.ttf"),
        Size=20,
        OnCommand=function(self)
            local numsongs = SONGMAN:GetNumSongs()
            local numgroups = SONGMAN:GetNumSongGroups()
            local numcourses = SONGMAN:GetNumCourses()

            self:settext(
                string.format(
                    THEME:GetString("ScreenTitleMenu", "%i Songs (%i Groups), %i Courses"),
                    SONGMAN:GetNumSongs(), SONGMAN:GetNumSongGroups(), SONGMAN:GetNumCourses()
                )
            ):Regen()
            self:zoom(
                zoomScale(
                    self:MainActor():GetWidth(),
                    self:MainActor():GetHeight(),
                    280,48
                )
            ):y(4)
        end
    },

    -- LoadModule("UI/UI.GenerateUIWithButtonAction.lua")(
    --     {
    --         Width=48,
    --         Height=48,
    --         CacheMouse = true,
    --         Pos={190, 0},
    --         Action=function( self )
    --             SCREENMAN:SetNewScreen("ScreenInitialSetup1")
    --         end,
    --         AddActors=Def.ActorFrame{
    --             Def.Sprite{
    --                 Texture = THEME:GetPathG("PlayMode", "Extended"),
    --                 OnCommand=function(self)
    --                     self:zoom(0.2)
    --                 end
    --             }
    --         }
    --     }
    -- )
}

local tryingConnection = false

-- Create the drop down menu for the gamemodes and a reload songs button
local leftPaneArea = Def.ActorFrame {
    InitCommand=function(self)
        self:xy(SCREEN_WIDTH * .25 - 100, 40)
    end,
    OnCommand=function(self)
		self:diffusealpha(0):easeoutexpo(0.2):diffusealpha(1)
	end,
    OffCommand=function(self)
		self:easeinexpo(0.25):addy(-100):diffusealpha(0)
	end,

    dropdown:Create()
}

t[#t+1] = LoadModule("UI/UI.GenerateUIWithButtonAction.lua")(
    {
        Width=48,
        Height=48,
        CacheMouse = true,
        Pos={SCREEN_WIDTH * .75 + 180, 40},
        Action=function( self )
            SCREENMAN:SetNewScreen("ScreenReloadSongs")
        end,
        AddActors=Def.ActorFrame{
            Def.Sprite{
                Texture = THEME:GetPathG("TitleMenuIcon/TitleScreen", "Refresh"),
                OnCommand=function(self)
                    self:zoom(0.6)
                end
            }
        }
    }
)..{
    OnCommand=function(self)
		self:diffusealpha(0):easeoutexpo(0.2):diffusealpha(1)
	end,
    OffCommand=function(self)
		self:easeinexpo(0.25):addy(-100):diffusealpha(0)
	end
}

if NETMAN then
    leftPaneArea[#leftPaneArea+1] = LoadModule("UI/UI.GenerateUIWithButtonAction.lua")(
        {
            Width=48,
            Height=48,
            CacheMouse = true,
            Pos={130, 0},
            ReturnAdjacentActorFrame = true,
            Action=function( self )
                if NETMAN:IsConnectionEstablished() or tryingConnection then return end
                self:playcommand("AttemptToConnect")
                -- SCREENMAN:SystemMessage("Connecting to server...")
                NETMAN:MachineLogin()
                tryingConnection = true
            end,
            AddActors=Def.ActorFrame{
                AttemptToConnectCommand=function(self)
                    self:GetChild("status"):diffusealpha(0)
                    self:GetChild("loadicon"):diffusealpha(.5)
                end,
                ResponseMachineLoginMessageCommand=function(self,params)
                    tryingConnection = false
                    -- lua.ReportScriptError( rin_inspect(params.Response) )
                    self:GetChild("status"):Load(
                        THEME:GetPathG("OnlineStatus", (params.Response ~= nil and "online" or "offline") )
                    ):diffusealpha(1)
                    self:GetChild("loadicon"):diffusealpha(0)
                    MESSAGEMAN:Broadcast("SystemNotification",{
                        Text= (params.Response and ("Connected") or "Could not connect") .. " to OutFox Online.",
                        Icon = THEME:GetPathG("OnlineStatus",params.Response and "online" or "offline")
                    })
                end,
                Def.Sprite{
                    Name="status",
                    Texture = THEME:GetPathG("OnlineStatus", (NETMAN:IsConnectionEstablished() and "on" or "off") .."line"),
                    OnCommand=function(self)
                        self:zoom(0.5)
                    end
                },

                Def.Sprite{
                    Name="loadicon",
                    Texture=THEME:GetPathG("OnlineStatus","connecting"),
                    Frames=Sprite.LinearFrames(3,1),
                    InitCommand=function(self)
                        self:zoom(0.5):diffusealpha(0)
                    end
                }
            }
        }
    )
end

t[#t+1] = leftPaneArea

return t
