return function( args )
	local PersonalBestBG = LoadModule("UI/UI.ButtonBox.lua")( args.Width + 1, 30, 2, nil, true )

	local itemwidth = 34
	local maxPossibleItemsInWidth = math.ceil((args.Width - 230) / (itemwidth))
	local possibleitems = 5

	local lastIndex = 1

	local StopMovingCursor = false

	local DiffScrollBottom = Def.ActorFrame{
		Name="DiffScrollBottom",
		UpdateDiffsCommand=function(self,params)
			local diff = params.Index-lastIndex
			local applyOffset = (diff < 0 or params.Force) and 1 or 0

			-- Only one item? don't even bother!
			if #params.Data == 2 then
				self:visible(false)
				return
			end

			self:visible(true)

			if params.Index == 2 and not params.Force and StopMovingCursor then
				-- Stop, we don't need to move anymore.
				return
			end

			StopMovingCursor = params.Index == 2

			for i = 1,possibleitems do
				local actor = self:GetChildAt(i)
				actor:visible( params.Data[params.Index+i+applyOffset] ~= nil )
				if params then
					local newdata = params.Data[params.Index+i+applyOffset]
					-- if i > 1 then
					actor:stoptweening()
					if newdata then
						local colboost = BoostColor( ColorDarkTone(GameColor.Difficulty[ newdata:GetDifficulty() ]), 0.95)
						-- Move the actors to their new spots.
						if (diff ~= 0) then
							actor:x( (itemwidth) * ((i-1)+((diff > 0) and 0 or -1)) )

							-- Before Pulse
							local pos1,pos2 = diff == -1 and 1 or 0, diff == -1 and 0 or 1
							if i == 4 then actor:diffusealpha(pos1) end
							if i == 1 then actor:diffusealpha(pos2) end

							-- After pulse
							actor:decelerate(0.2):x( (itemwidth) * ((i-1)+((diff > 0) and -1 or 0)) )

							if i == 4 then actor:diffusealpha(pos2) end
							if i == 1 then actor:diffusealpha(pos1) end
						end
						actor:GetChild("Diff"):diffuse( colboost )
						actor:GetChild("Meter"):diffuse( ColorLightTone(GameColor.Difficulty[ newdata:GetDifficulty() ]) )
						:settext( newdata:GetMeter()  )
					end
					-- Last item, if there's more data after this, convert this item to a +.
					if i >= 3 and params.Data[4+1] then
						if i == (4-applyOffset) then
							actor:GetChild("Diff"):diffuse( color("#555555") )
							actor:GetChild("Meter"):diffuse( Color.White ):settext( "+" )
						else
							if i >= 4 then
								actor:visible(false)
							end
							-- actor:GetChild("Meter"):settext("a")
						end
					end
					-- actor:GetChild("Meter"):settext(i)
				end
			end
		end
	}

	for i = 0,possibleitems-1 do
		DiffScrollBottom[#DiffScrollBottom+1] = Def.ActorFrame{
			OnCommand=function(self)
				local af = self:GetParent():GetParent()
				self:x( (itemwidth) * (i) ):zoom(0.8)
			end,

			Def.Quad{ Name="Diff", InitCommand=function(self) self:zoomto(itemwidth,28) end },
			Def.BitmapText{ Font="_Bold", Name="Meter" }
		}
	end

	-- local DynChartNames = LoadModule("Steps.GetDynamicChartName.lua")
	local function GenerateDiffTextInfo( self, Diff )
		local match = true
		if not Diff or not Diff.GetDifficulty then match = false end

		if not match then
			self:GetChild("Name"):settext("")
			self:GetChild("Meter"):settext("")
			if self:GetChild("Author") then
				self:GetChild("Author"):settext( "" )
			end
			if self:GetChild("Diff") then
				self:GetChild("Diff"):stoptweening():linear(0.1):diffuse( Alpha(Color.White,0.6) )
			end
			return
		end

		local colboost = BoostColor( GameColor.Difficulty[ Diff:GetDifficulty() ], 0.95)

		local newColor = Diff:IsAutogen() and Color.Yellow or Color.White
		self:GetChild("Name"):diffuse(newColor)
		:settext( THEME:GetString("CustomDifficulty",ToEnumShortString(Diff:GetDifficulty())) )

		if self:GetChild("Author") then
			self:GetChild("Author"):settext( Diff:GetAuthorCredit() or Diff:GetDescription() )
			:diffuse( newColor )
		else
			self:GetChild("Name"):diffuse( colboost )
		end
		self:GetChild("Meter"):settext( Diff:GetMeter() ):diffuse(newColor)
		if self:GetChild("Diff") then
			self:GetChild("Diff"):stoptweening():linear(0.1):diffuse( colboost )
		end
	end
	local t = Def.ActorFrame{
		Name="DiffSelector",
		InitCommand=function(self)
			self:y(118):diffusealpha(0)
		end,
		PlayerSwitchedStepMessageCommand=function(self,params)
			if params.Player ~= args.Player then return end
			if not GAMESTATE:IsPlayerEnabled(args.Player) then return end
			if type(params.Song) == "string" then
				-- lua.ReportScriptError("not song")
				self:stoptweening():linear(0.1):diffusealpha(0)
				return
			end
			-- lua.ReportScriptError("Changed")

			self:stoptweening():linear(0.1):diffusealpha(1)

			local curSteps = params.Song[ params.Index ]
			local colboost = BoostColor( ColorDarkTone(GameColor.Difficulty[ curSteps:GetDifficulty() ]), 0.95)
			self:GetChild("MainBG"):stoptweening():linear(0.1):diffuse( BoostColor(colboost, 2.2)  )

			PersonalBestBG:ChangeColor( ColorLightTone(colboost), ColorDarkTone(colboost) )

			self:GetChild("Diff3"):GetChild("DiffScrollBottom"):playcommand("UpdateDiffs",{Data=params.Song,Index = params.Index, Force = params.Force or false})

			lastIndex = params.Index

			local stats = PROFILEMAN:GetProfile(args.Player):GetHighScoreListIfExists(
				params.Song[1], curSteps
			)

			if stats then
				local highscores = stats:GetHighScores()
				if #highscores > 0 then
					self:GetChild("HighScore"):GetChild("Score"):settext(
						FormatPercentScore(stats:GetHighScores()[1]:GetPercentDP())
					)
				else
					self:GetChild("HighScore"):GetChild("Score"):settext( "-N/A-" )
				end
			else
				self:GetChild("HighScore"):GetChild("Score"):settext( "-N/A-" )
			end

			GenerateDiffTextInfo( self:GetChild("Diff1"), params.Song[ params.Index-1 ] )
			GenerateDiffTextInfo( self:GetChild("Diff2"), params.Song[ params.Index ] )
			GenerateDiffTextInfo( self:GetChild("Diff3"), params.Song[ params.Index+1 ] )
		end
	}

	t[#t+1] = LoadModule("Object.GenCornerBox.lua"){
		Width = args.Width*2 - 50,
		Height = 60,
		Corners = {false,false,false,false}
	}..{
		Name="MainBG",
		OnCommand=function(self)
			self:finishtweening():zoom(0.5)
		end
	}

	local HighlightColorSize = args.Width*.45
	local OtherDiffWidth = args.Width*.65

	t[#t+1] = Def.ActorFrame{
		InitCommand=function(self) self:y( -self:GetParent():GetChild("MainBG"):GetZoomedHeight() + 14 ) end,
		Name="Diff1",
		LoadModule("UI/UI.ClickArea.lua"){
			Width = args.Width,
			Height = 30,
			Action = function(self)
				args.DiffMoveFunc( args.Player, -1 )
			end
		},
		LoadModule("Object.GenCornerBox.lua"){
			Width = args.Width*2 - 50,
			Height = 10,
			Corners = {false,true,false,false}
		}..{
			Name="Diff",
			OnCommand=function(self) self:finishtweening():zoom(0.5) end
		},
		Def.BitmapText{ Font="_Bold", Name="Name",
			InitCommand=function(self)
				self:halign(0):x( -args.Width*.5 + 40 )
				:uppercase(true):zoom(0.8)
			end
		},
		Def.BitmapText{ Font="_Bold", Name="Meter", InitCommand=function(self) self:x( args.Width*.5 - 30 ) end },

		Def.BitmapText{
			Font="Common Normal",
			Text="&MENUUP;",
			InitCommand=function(self)
				self:zoom(0.6):x( -args.Width*.5 + 15 )
			end
		}
	}
	t[#t+1] = Def.ActorFrame{
		Name="Diff2",
		Def.BitmapText{
			Font="_Bold",
			Name="Name",
			InitCommand=function(self)
				self:halign(0):xy( -args.Width*.5 + 20, -8 ):uppercase(true):zoom(1):diffusealpha(0.8)
			end
		},
		Def.BitmapText{
			Font="_kurinto sans med/20px",
			Name="Author",
			InitCommand=function(self)
				self:halign(0):xy( -args.Width*.5 + 20, 8 ):zoom(0.6):diffusealpha(0.8)
			end
		},
		Def.BitmapText{ Font="_Large", Name="Meter", InitCommand=function(self) self:halign(1):x( args.Width*.5 - 20 ):zoom(1) end }
	}
	t[#t+1] = Def.ActorFrame{
		InitCommand=function(self) self:y( self:GetParent():GetChild("MainBG"):GetZoomedHeight() + 18 ) end,
		Name="Diff3",
		LoadModule("Object.GenCornerBox.lua"){
			Width = args.Width*2 - 50,
			Height = 10,
			Corners = {false,false,false,true}
		}..{
			Name="Diff",
			OnCommand=function(self) self:finishtweening():zoom(0.5) end
		},
		LoadModule("UI/UI.ClickArea.lua"){
			Width = args.Width,
			Height = 30,
			Action = function(self)
				args.DiffMoveFunc( args.Player, 1 )
			end
		},
		Def.BitmapText{ Font="_Bold", Name="Name",
			InitCommand=function(self)
				self:halign(0):x( -args.Width*.5 + 40 )
				:uppercase(true):zoom(0.8)
			end
		},
		Def.BitmapText{ Font="_Bold", Name="Meter", InitCommand=function(self) self:x( args.Width*.5 - 30 ) end },

		DiffScrollBottom,

		Def.BitmapText{
			Font="Common Normal",
			Text="&MENUDOWN;",
			InitCommand=function(self)
				self:zoom(0.6):x( -args.Width*.5 + 15 )
			end
		}
	}

	t[#t+1] = Def.ActorFrame{
		InitCommand=function(self) self:y( self:GetParent():GetChild("MainBG"):GetZoomedHeight() - 14 ) end,
		Name="HighScore",
		PersonalBestBG:Create(),
		Def.Text{
			Font=THEME:GetPathF("","IBMPlexSans-Bold.ttf"),
			Size=20,
			Text=THEME:GetString(Var "LoadingScreen", "PersonalBest"),
			InitCommand=function(self)
				self:xy( -args.Width*.5 + 10, 3 ):zoom(0.6)
				:MainActor():halign(0)
				self:StrokeActor():halign(0)
			end
		},

		Def.BitmapText{ Font="_Bold", Name="Score", InitCommand=function(self) self:halign(1):x( args.Width*.5 - 70 ):zoom(0.8) end }
	}

	return t
end
