return function( args )
	-- This bit will be cleaned later
	local pn = args.Player
	local DiffBarStartPosition = SCREEN_CENTER_Y + 130
	local getTiming = LoadModule("Options.ReturnCurrentTiming.lua")
	local curTiming = getTiming().Name

	-- Fetch the current player's target from a selected score.
	if not GAMESTATE:Env()["ScoreTargets"] then
		GAMESTATE:Env()["ScoreTargets"] = {}
	end
	local PlayerTarget = GAMESTATE:Env()["ScoreTargets"][pn] or nil

	local BGWidth = args.Width
	local BGHeight = 390
	local scoresLoaded = false

	local NoScoresText = THEME:GetString("ScreenPlayerOptionsLua","NoScoresAvailable")

	local t = Def.ActorFrame{
		Name="Leaderboard",
		InitCommand=function(self)
			self:y( DiffBarStartPosition ):visible(false):diffusealpha(0)
		end,
	}

	local indexPickForPace = 1

	local function IsScoreTarget( self )
		if not self.ScoreInfo then
			self:diffuse( Alpha(Color.White, 0.5) )
			return
		end
		if not PlayerTarget then self:diffuse(Color.White) return end

		self:diffuse(self.ScoreInfo.score == PlayerTarget.score and Color.Green or Color.White)
	end

	local function MoveIndex( self, offset )
		if not scoresLoaded then return end

		local old = indexPickForPace
		indexPickForPace = indexPickForPace + offset
		
		if indexPickForPace > 10 then indexPickForPace = 1 end
		if indexPickForPace < 1 then indexPickForPace = 10 end
		
		IsScoreTarget( self:GetChild(old) )
		self:GetChild(indexPickForPace):diffuse( Color.Orange )
	end

	local lastSteps
	local isEnabled = false
	local ScoreRequest
	local temp_curChartProcess
	-- Generate score template
	local scores = Def.ActorFrame{
		PlayerSwitchedStepMessageCommand=function(self,params)
			if type(params.Song) == "string" then
				self:visible(false)
			end
		end,
		UpdateOnlineChartInfoMessageCommand=function(self,params)
			if lastSteps == params.Steps then return end

			self.tempParams = params
			self:stoptweening():visible(true)

			if not isEnabled or not params.Toggle then return end

			self:GetChild("NoScores"):settext("")
			self:GetChild("loadicon"):visible(true)
			for i = 1,10 do
				self:GetChild(i):diffusealpha(0)
			end

			curTiming = getTiming().Name

			self:sleep(0.25):queuecommand("BeginRequest")
		end,
		BeginRequestCommand=function(self)
			LoadModule("Online.GetScoreData.lua")(
				GAMESTATE:GetCurrentSong(),
				self.tempParams.Steps,
				curTiming
			)
		end,
		ResponseHighScoresForChartMessageCommand=function(self,params)
			self.tempParams.response = params.Response
			-- lua.ReportScriptError(rin_inspect(params.Response))
			self:playcommand("UpdateChartInfo",self.tempParams)
		end,
		UpdateChartInfoCommand=function(self,params)
			-- lua.ReportScriptError("Chart Info")
			-- Also manipulate DiffBG here to update it's coloring.
			local cd = GetCustomDifficulty(params.Steps:GetStepsType(), params.Steps:GetDifficulty())
			
			local scoredata = nil
			-- XXX: Currently processing the JSON decode here as a quick test.
			-- We *should* be able to process this already before sending it as a command,
			-- but will require some manual Lua manipulation engine-side.
			-- Some matches may hit a nil value, i am aware.
			if params.response ~= nil then
				-- lua.ReportScriptError( rin_inspect(newdata) )
				local newdata = params.response
				if #newdata.scores ~= 0 then
					scoredata = newdata.scores
				end
			end
			lastSteps = self.tempParams.Steps
			-- lua.ReportScriptError(rin_inspect(scoredata))
			self:GetChild("loadicon"):visible(false)
			if not scoredata then
				self:GetChild("NoScores"):y( 20 ):settext( NoScoresText ):visible(true)
				for i = 1,10 do
					self:GetChild(i):diffusealpha(0)
				end
				return
			end


			scoresLoaded = true

			self:GetChild("NoScores"):y(BGHeight*.5 - 42):settext( curTiming )

			local function FormatScore(score)
				if curTiming == "DDR" then
					return score
				end
				return FormatPercentScore(score)
			end

			for i = 1,10 do
				self:GetChild(i):diffusealpha(0):linear(0.1):diffusealpha(1)
				self:GetChild(i):GetChild("BG"):visible(true)
				self:GetChild(i).ScoreInfo = scoredata[i]
				local username = scoredata[i] and scoredata[i].username or nil
				local score = scoredata[i] and scoredata[i].score or 0
				local date = scoredata[i] and scoredata[i].date or nil
				
				local row = self:GetChild(i):GetChildren()
				self:GetChild(i):diffusealpha( scoredata[i] and 1 or 0.5 )

				IsScoreTarget( self:GetChild(i) )

				row.PlayerName:settext( username or "-N/A-" )
				row.Date:settext( "---" )
				if date then
					local year, month, day = date:match("([%d]+)-([%d]+)-([%d]+)")
					row.Date:settext( string.format("%d/%d/%d", year, month, day) )
				end
				row.Score:settext( FormatScore(score) )
			end

			MoveIndex( self, 0 )
		end,
		EnableLeaderboardCommand=function(self,params)
			if params.Player ~= pn then return end
			isEnabled = params.Toggle
			if not params.Toggle then return end
			-- if lastSteps == params.Steps then return end
			-- lastSteps = params.Steps
			
			-- self:playcommand("UpdateChartInfo",params)
		end,

		MoveSelectionForPaceCommand=function(self,params)
			MoveIndex( self, params.offset )
		end,

		SetIndexAsTargetScoreCommand=function(self,params)
			if not self:GetChild(indexPickForPace).ScoreInfo then return end
			
			GAMESTATE:Env()["ScoreTargets"][pn] = self:GetChild(indexPickForPace).ScoreInfo
			PlayerTarget = GAMESTATE:Env()["ScoreTargets"][pn]

			for i = 1,10 do
				IsScoreTarget( self:GetChild(i) )
			end
		end,

		Def.BitmapText{
			Name="NoScores", Font="Common Normal", Text=NoScoresText,
			InitCommand=function(self)
				self:y( BGHeight*.5 - 80  )
			end
		},

		Def.Sprite{
			Name="loadicon",
			Texture=THEME:GetPathG("Loading","icon"),
			InitCommand=function(self)
				self:spin():y( 30 ):zoom(0.3):effectmagnitude(0,0,400)
			end
		}
	}

	for i = 1,10 do
		scores[#scores+1] = Def.ActorFrame{
			Name=i,
			InitCommand=function(self)
				self:y( -130 + 28 * (i-1) )

				-- self:GetChild("BG"):zoomto( BGWidth - 40, 26  ):diffuse(Color.Green)
				:diffusealpha(0.5)
				self:GetChild("Index"):x( -BGWidth*.5 + 45 ):halign(1):maxwidth(36)
				self:GetChild("PlayerName"):x( -BGWidth*.5 + 54 ):zoom(0.8):halign(0)
				self:GetChild("Score"):x( BGWidth*.5 - 20 ):zoom(0.8):halign(1)
				self:GetChild("Date"):x(30):zoom(0.8)
			end,
			--
			LoadModule("UI/UI.ButtonBox.lua")( BGWidth - 30, 25, 2, nil )..{
				Name="BG",
			},
			Def.BitmapText{ Name="Index", Text=i, Font="Common Normal" },
			Def.BitmapText{ Name="PlayerName", Text="-N/A-", Font="Common Normal" },
			Def.BitmapText{ Name="Date", Text="---", Font="Common Normal" },
			Def.BitmapText{ Name="Score", Text=FormatPercentScore(0), Font="Common Normal" },
			LoadModule( "UI/UI.ClickArea.lua" ){
				Width = BGWidth - 30,
				Height = 26,
				ReturnAdjacentActorFrame = true,
				Action = function(self)
					if not isEnabled or not scoresLoaded then return end

					IsScoreTarget( self:GetParent():GetChild(indexPickForPace) )
					indexPickForPace = i
					self:GetParent():playcommand("MoveSelectionForPace",{offset=0})
					self:GetParent():playcommand("SetIndexAsTargetScore",{ IsTouch=true })
				end
			}..{ Name="ClickArea" }
		}
	end

	t[#t+1] = scores

	-- Test for text fading actortype.
	local TextScrollTitle = LoadModule("Text.FadeSlide.lua"){
		Width = BGWidth-50,
		Font = "_Bold",
		Height = 30,
	}

	local TextScrollSubTitle = LoadModule("Text.FadeSlide.lua"){
		Width = (BGWidth+40),
		Height = 30,
		Font = "_Bold"
	}

	return t
end