local DiffBoxHandle
local itemwidth = 44

return function( args )
	local WheelItemWidth = args.Width
	-- Calculate the ammount of possible items that can fit on the box.
	-- Note: When new players join, the limit must change so the + box changes.
	local moreThanAPlayer = GAMESTATE:GetNumPlayersEnabled() > 1
	local maxPossibleItemsInWidth = math.ceil(WheelItemWidth / (itemwidth+4))
	local possibleitems = 13
	local DiffBoxHandler
	local DiffScroll = Def.ActorFrame{
		InitCommand=function(self)
			DiffBoxHandler = self
		end,
		UpdateDiffsCommand=function(self,params)
			for i = 1,possibleitems do
				local actor = self:GetChildAt(i)
				actor:visible( params ~= nil and params.Data[i+1] ~= nil )
				if params then
					if params.Data[i+1] then
						local colboost = BoostColor( ColorDarkTone(GameColor.Difficulty[ params.Data[i+1]:GetDifficulty() ]), 0.95)
						actor:GetChild("Diff"):diffuse( colboost )
						actor:GetChild("Meter"):diffuse( ColorLightTone(GameColor.Difficulty[ params.Data[i+1]:GetDifficulty() ]) )
						:settext( params.Data[i+1]:GetMeter()  )
					end


					-- Last item, if there's more data after this, convert this item to a +.
					if i >= maxPossibleItemsInWidth and params.Data[maxPossibleItemsInWidth+1] then
						if i == maxPossibleItemsInWidth then 
							actor:GetChild("Diff"):diffuse( color("#555555") )
							actor:GetChild("Meter"):diffuse( Color.White ):settext( "+" )
						else
							actor:visible(false)
						end
					end
				end
			end
		end
	}

	for i = 0,possibleitems-1 do
		DiffScroll[#DiffScroll+1] = Def.ActorFrame{
			OnCommand=function(self)
				local af = self:GetParent():GetParent()
				self:x( -af:GetWidth() *.5 + 180 + (itemwidth+4) * i )
			end,

			Def.Quad{ Name="Diff", InitCommand=function(self) self:zoomto(itemwidth,28) end },
			Def.BitmapText{ Font="_Bold", Name="Meter" }
		}
	end

	local TouchButtons = Def.ActorFrame{
		Condition = args.Touch,
		LoadModule("UI/UI.GenerateUIWithButtonAction.lua"){
			Width = 90,
			Height = 48,
			Pos = {0, -80},
			ActionIsAfterLifting = true,
			Action = function( self )
				args.MoveFunction( -1 )
			end,
			AddActors = Def.ActorFrame{
				Def.Sprite{
					Texture=THEME:GetPathG("","UI/Back"),
					InitCommand=function(self)
						self:rotationz(0)
					end
				}
			}
		},

		LoadModule("UI/UI.GenerateUIWithButtonAction.lua"){
			Width = 90,
			Height = 48,
			Pos = {0, 80},
			ActionIsAfterLifting = true,
			Action = function( self )
				args.MoveFunction( 1 )
			end,
			AddActors = Def.ActorFrame{
				Def.Sprite{
					Texture=THEME:GetPathG("","UI/Back"),
					InitCommand=function(self)
						self:rotationz(180)
					end
				}
			}
		},

		-- ???/TODO: Something has gone wrong with the click module's position process.
		-- Zooming is the culprit, as it is performed on the base actorframe. Either shrink items on the standard ActorFrame
		-- size, or make an algorithm that can process changes on the draw size of adjacent childs.
		LoadModule("UI/UI.ClickArea.lua"){
			Width = 90,
			Height = 100,
			ActionIsAfterLifting = true,
			ReturnAdjacentActorFrame = true,
			Position = function(self)
				local af = self:GetParent():GetParent()
				self:x( -af:GetWidth() * .5 )
			end,
			Action = function( self )
				args.StartFunction()
			end,
		}
	}

	local ArtistFade = LoadModule("Text.FadeSlide.lua"){
		Width = WheelItemWidth * 2 - 60, Font = "_Kurinto Medium", Height = 30,
		SpeedFactor = 0.1
	}

	return Def.ActorFrame{
		Name="SongInfoBox",
		InitCommand=function(self)
			self:SetWidth( WheelItemWidth + 60 ):SetHeight(100)
			:xy( SCREEN_CENTER_X, SCREEN_CENTER_Y )
			self:GetChild("BGItem"):diffuse( GameColor.Custom["SongPanelStripe"])
			self:GetChild("BGHighlight"):diffuseramp()
			:effectcolor1(ColorDarkTone( GameColor.Custom["SongPanelHighlight"])):effectcolor2( GameColor.Custom["SongPanelHighlight"]):effectclock('beatnooffset')

			ArtistFade:ApplyToText( function(self)
				self:shadowlength(4)
			end )

			-- Add a wrapper state to this actorframe as I can't seem to control it's tweens anymore.
			self:AddWrapperState()
		end,
		CancelCommand=function(self)
			self:GetWrapperState(1):sleep(0.2):easeoutexpo(0.25):diffusealpha(0)
		end,
		UpdateSizeCommand=function(self,params)
			maxPossibleItemsInWidth = math.ceil((params.Width + 80) / (itemwidth+4))

			self:SetWidth( params.Width + 60 )
			if params.Height then
				self:SetHeight( params.Height )
			end
			self:RunCommandsRecursively(
				function(self)
					self:stoptweening():easeoutexpo(0.2):playcommand("On")
				end
			)
		end,
		UpdateSongInfoCommand=function(self,params)
			local c = self:GetChildren()

			c.Image:playcommand("Load",params)
			c.FallbackJacket:playcommand("Load",params)
			c.BannerView:GetChildAt(1):playcommand("Load",params)
			local SInfo = self:GetChild("SongInfo"):GetChildren()
			if params and type(params.Data) ~= "string" then
				SInfo.Title:settext( params.Data[1]:GetDisplayMainTitle() )
				ArtistFade:SetText( params.Data[1]:GetDisplayArtist() )
				SInfo.SubTitle:settext( params.Data[1]:GetDisplaySubTitle() )
				
				local subsAvailable = params.Data[1]:GetDisplaySubTitle() ~= ""
				SInfo.SubTitle:stoptweening():easeoutexpo(0.2):diffusealpha( subsAvailable and 1 or 0 )
				SInfo.Title:stoptweening():easeoutexpo(0.2):zoom( subsAvailable and 0.7 or 0.8 )
				SInfo.Artist:stoptweening():easeoutexpo(0.2):y( params.Data[1]:GetDisplaySubTitle() ~= "" and 0 or -14 )

				-- Handle how the BPM will be displayed on the metadata viewer.
				local bpms = params.Data[1]:GetDisplayBpms()
				-- The values we get from this value are not rounded, so we have to take care of that.
				bpms[1] = math.floor(bpms[1]+0.5)
				bpms[2] = math.floor(bpms[2]+0.5)
				local toshow = ""
				if bpms[1] == bpms[2] then
					toshow = bpms[1]
				else
					toshow = table.concat(bpms, "-")
				end
				c.BPM:visible(true):GetChild("Value"):settext( toshow )
				c.Length:visible(true):GetChild("Value"):settext( SecondsToMMSS(params.Data[1]:MusicLengthSeconds()) )

				DiffBoxHandler:playcommand("UpdateDiffs",params)

				if params.visSearch and params.visSearch.value then
					local states = TF_WHEEL.SearchStates
					-- If mode isn't set, then we might just bee looking at a title search as well..
					local touse = {
						["Title"] = params.Data[1]:GetDisplayMainTitle(),
						["SubTitle"] = params.Data[1]:GetDisplaySubTitle(),
						["Artist"] = params.Data[1]:GetDisplayArtist(),
					}
					
					-- Highlight the part of the text that was found in the search.
					for i,v in pairs( touse ) do
						-- Does the user want this search?
						if states[ToLower(i)] then
							local matches, start = string.find( ToLower(v), params.visSearch.value )
							local searchlength = string.len(params.visSearch.value)
							-- We only need to do this if we have any kind of match.
							if matches then
								if i == "Artist" then
									ArtistFade:ApplyToText(
										function(self)
											self:AddAttribute( matches-1, { Length = searchlength, Diffuse = GameColor.Custom["MenuButtonBorder"] } )
										end
									)
								else
									self:GetChild("SongInfo"):GetChild(i):AddAttribute( matches-1, { Length = searchlength, Diffuse = GameColor.Custom["MenuButtonBorder"] } )
								end
							end
						end
					end
				end
			else
				-- It's a group, show different info.
				SInfo.Title:settext( params.Data )
				ArtistFade:SetText( "" )
				SInfo.SubTitle:settext("")
				
				c.BPM:visible(false)
				c.Length:visible(false)
				
				DiffBoxHandler:playcommand("UpdateDiffs",nil)
			end
		end,

		LoadActor(THEME:GetPathB("_frame","3x3"), THEME:GetPathG("SelectMusic/SongPanel","base"), WheelItemWidth, 3)..{
			Name="BGItem",
		},

		Def.Quad{
			OnCommand=function(self)
				local af = self:GetParent()
				self:diffuse(color("#131429")):visible(false)
				:zoomto( af:GetWidth(), af:GetHeight() )
			end
		},
		Def.Quad{
			OnCommand=function(self)
				local af = self:GetParent()
				self:diffuse(color("#012837")):visible(false)
				:zoomto( af:GetWidth(), af:GetHeight() * .25 )
				:valign(1)
				:y( af:GetHeight()*.5 )
			end
		},

		Def.ActorFrameTexture{
			Name="BannerView",
			InitCommand=function(self)
				-- local af = self:GetParent():GetParent():GetParent()
				self:SetWidth(220):SetHeight(134):EnableAlphaBuffer(false)
				:Create()
			end,

			Def.Sprite{
				InitCommand=function(self)
					self:align(1,0)
					:x( 220 )
				end,
				LoadCommand=function(self,param)
					-- Check if its a song.
					local imageFound = false
					if type(param.Data) ~= "string" then
						-- It is, Load banner
						if param.Data[1]:GetBannerPath() then
							self:LoadFromCached("banner",param.Data[1]:GetBannerPath())
							imageFound = true
						end
					else
						-- It's a group, Check if it has a banner.
						if SONGMAN:GetSongGroupBannerPath(param.Data) ~= "" then
							-- It does, Load it.
							self:LoadFromCached("banner",SONGMAN:GetSongGroupBannerPath(param.Data))
							imageFound = true
						end
					end
					
					self:visible(imageFound):zoom(TF_WHEEL.Resize(self:GetWidth(),self:GetHeight(),250,100))
				end
			}
		},

		Def.Sprite{
			OnCommand=function(self)
				self:SetTexture( self:GetParent():GetChild("BannerView"):GetTexture() )

				local af = self:GetParent()
				self:xy(af:GetWidth() * .5 + 20, -af:GetHeight()*.5 ):valign(0):halign(1)
				:diffusealpha(0.3):fadeleft(0.3):cropbottom(0.45)
			end
		},

		Def.Quad{
			Name="JacketBG",
			OnCommand=function(self)
				local af = self:GetParent()
				self:diffuse(color("#002339"))
				:zoomto( 90, 90 )
				:x( -af:GetWidth() * .5 + 55 )
			end
		},

		Def.Sprite{
			Name="FallbackJacket",
			Texture=THEME:GetPathG("Common fallback","jacket"),
			OnCommand=function(self)
				local af = self:GetParent()
				self:x( -af:GetWidth() * .5 + 55 ):zoomto(90,90)
			end,
			LoadCommand=function(self,params)
				local isASong = type(params.Data) == "table"
				local function HasAnImage(song)
					if song:GetBannerPath() then
						return song:GetBannerPath() ~= ""
					end
					if song:GetJacketPath() then
						return song:GetJacketPath() ~= ""
					end
					return false
				end
				self:visible( isASong and not HasAnImage(params.Data[1]) )
			end
		},

		LoadActor(THEME:GetPathB("_frame","3x3"), THEME:GetPathG("SelectMusic/SongPanel","highlight"), WheelItemWidth, 3)..{
			Name="BGHighlight",
		},

		Def.ActorFrame{
			Name="SongInfo",
			OnCommand=function(self)
				local af = self:GetParent()
				self:xy(-(af:GetWidth() * .5) + 105, -4 )
			end,
			Def.BitmapText{
				Font="_Kurinto Semibold",
				Name="Title",
				InitCommand=function(self) self:y( -30 ):halign(0):zoom(0.8):shadowlength(2) end
			},
			Def.BitmapText{
				Font="_Kurinto Medium",
				Name="SubTitle",
				InitCommand=function(self) self:y( -14 ):halign(0):zoom(0.6):shadowlength(2) end
			},
			ArtistFade:Create()..{
				Name="Artist",
				InitCommand=function(self)
					self:xy( WheelItemWidth*.5 - 15, -14 ):zoom(0.5)
				end
			}
		},

		-- BPM Display
		Def.ActorFrame{
			Name="BPM",
			OnCommand=function(self)
				local af = self:GetParent()
				self:diffuse(color("#9DE8ED"))
				:xy(-(af:GetWidth() * .5) + 97 + 10, 13 )
			end,
			Def.Text{
				Font=THEME:GetPathF("","IBMPlexSans-Bold.ttf"),
				Size=40,
				Text=ToUpper(THEME:GetString("MusicWheel","BpmText")),
				InitCommand=function(self)
					self:zoom(0.25):y(3):skewx(-0.2)
					:diffusealpha(0.8):shadowlength(2)
					:MainActor():halign(0):shadowlength(2)
					self:StrokeActor():halign(0):shadowlength(2)
				end
			},
			Def.BitmapText{
				Font="_Bold",
				Name="Value",
				InitCommand=function(self)
					self:xy(25, -.5):zoom(0.6):halign(0):shadowlength(2)
				end
			},
		},

		-- Song Length
		-- BPM Display
		Def.ActorFrame{
			Name="Length",
			OnCommand=function(self)
				local af = self:GetParent()
				self:diffuse(color("#9DE8ED"))
				:xy(-(af:GetWidth() * .5) + 190 + 10, 13 )
			end,
			Def.Text{
				Font=THEME:GetPathF("","IBMPlexSans-Bold.ttf"),
				Size=40,
				Text=ToUpper(THEME:GetString("MusicWheel","LengthText")),
				InitCommand=function(self)
					self:zoom(0.25):y(3):skewx(-0.2)
					:diffusealpha(0.8):shadowlength(2)
					:MainActor():halign(0):shadowlength(2)
					self:StrokeActor():halign(0):shadowlength(2)
				end
			},
	
			Def.BitmapText{
				Font="_Bold",
				Name="Value",
				OnCommand=function(self)
					self:xy( self:GetParent():GetChildAt(1):MainActor():GetZoomedWidth()*.25 + 5 , -.5 )
					:zoom(0.6):halign(0):shadowlength(2)
				end
			},
		},

		Def.Sprite{
			Name="Image",
			OnCommand=function(self)
				local af = self:GetParent()
				self:x( -af:GetWidth() * .5 + 55 )
			end,
			LoadCommand=function(self,param)
				-- Check if its a song.
				if type(param.Data) ~= "string" then
					-- It is, Load banner
					local found = false

					if param.Data[1]:GetJacketPath() then
						self:LoadFromCached("jacket",param.Data[1]:GetJacketPath())
						found = true
					else
						if param.Data[1]:GetBannerPath() then
							self:LoadFromCached("banner",param.Data[1]:GetBannerPath())
							found = true
						end
					end
					self:visible(found)
				else
					-- It's a group, Check if it has a banner.
					if SONGMAN:GetSongGroupBannerPath(param.Data) ~= "" then
						-- It does, Load it.
						self:visible(true):LoadFromCached("banner",SONGMAN:GetSongGroupBannerPath(param.Data))
					else
						-- It doesnt, Hide the banner.
						self:visible(false)
					end
				end
				
				self:zoom(TF_WHEEL.Resize(self:GetWidth(),self:GetHeight(),90,90))
			end
		},

		Def.Sprite{
			Condition = args.Touch,
			Texture=THEME:GetPathG("SelectMusic/Play","Triangle"),
			OnCommand=function(self)
				local af = self:GetParent()
				self:x( -af:GetWidth() * .5 + 55 )
				:zoom(2):diffusealpha(0.7):blend('add')
				:diffuseramp()
				-- :effectcolor1(ColorDarkTone( GameColor.Custom["SongPanelHighlight"]))
				:effectcolor1( color("#ffffff44") )
				:effectclock('beatnooffset')
			end
		},

		DiffScroll..{
			OnCommand=function(self)
				local af = self:GetParent()
				self:zoom(0.6):xy( -af:GetWidth()*.2 + 10, af:GetHeight()*.5 - 13 )
			end
		},

		TouchButtons
	}
end