-- This is the version of the wheel for > 4:3 screens. For the vertical screen version, check
-- the Wheel.Song.Vertical.lua file.

-- TODO: Crazy idea but, make a manager that handles the back bones of the wheel, and these modules just control
-- the visual aspects from it. This would reduce the clutter of having the same logic form from them.
local RepeatRateTime = 1 / THEME:GetMetric( Var "LoadingScreen", "RepeatRate" )
local aspectRatio = GetScreenAspectRatio()
local isEventMode = GAMESTATE:IsEventMode()
local doneWithInput = false
local IsVerticalScreen = aspectRatio < 1
local inSearchMode = false
local enteredSearch = false
local holdingSelect = false

local needsSDVersion = aspectRatio < 1.34

local UseTwoStepDiff = LoadModule("Config.Load.lua")("UseTwoStepDiff","/Save/OutFoxPrefs.ini")
local CanUseTouchControls = LoadModule("Config.Load.lua")("UseTouchControls","/Save/OutFoxPrefs.ini")

-- Useful function for determine a range and limiting it without overflowing
local function LimitRange(Val, Start, Endv, Min, Max)
    -- Unfortunately, the scale function available does not properly limit the value
    local EndVal = scale(Val, Start, Endv, Min, Max)
	return clamp( EndVal, Min, Max )
end

local PlayerAreaWidth = LimitRange(aspectRatio,1.60,1.77,350,360)

local WheelItemWidth = GAMESTATE:GetNumPlayersEnabled() == 1 and (SCREEN_WIDTH*.8 - PlayerAreaWidth) or clamp( scale(aspectRatio,1.60,1.77,300,360), 300, 700 )

if needsSDVersion then
	PlayerAreaWidth = 300
	WheelItemWidth = SCREEN_WIDTH*.8 - PlayerAreaWidth - 20
end

local Songs
local GroupsAndSongs
local PlayerPage = {
	[PLAYER_1] = { Index = 1 },
	[PLAYER_2] = { Index = 1 }
}

local function AnyPlayerIsOnOptions()
	for player in ivalues( PlayerNumber ) do
		if PlayerPage[player].MenuOpen then
			return true
		end
	end
	return false
end

local ThreeButtonComp = PREFSMAN:GetPreference("ThreeKeyNavigation")

-- Load up the set of utilities for this wheel.
local ModuleUtils = {
	GroupSet = LoadModule("Wheel/Group.Sort.lua"),
	DynChartNames = LoadModule("Steps.GetDynamicChartName.lua"),
	SongLoader = LoadModule("Wheel/Songs.Loader.lua")
}

local LastGroup = ""
-- Use this toggle to show jackets on the wheel item.
local showJackets = true

local NotEnoughStages = THEME:GetString(Var "LoadingScreen", "NotEnoughStagesToPlay")
local SearchSongLabel = THEME:GetString(Var "LoadingScreen", "SearchSongs")
local BeginSearch,RestoreWheelFromSearch

local availablesortingmodes = {
	Group = { "group", "title", "artist" },
	Song = { "title", "artist", "genre", "bpm", "length" }
}

if not m_styleindex then m_styleindex = 1 end
if not m_SortingModes then m_SortingModes = { Group = 1, Song = 1 } end

local function getSortingModeFromType( type )
	return availablesortingmodes[type][ m_SortingModes[type] ]
end

local visSearch = {}

local function fetchSearchModeFromFlag()
	local _, _, key, value
	-- Special case, was it pack/song? 
	_, _, key, value = string.find(CurSearch, "(.+)/(.+)")
	if not key and not value then
		-- No, then revert to flag search.
		_, _, key, value = string.find(CurSearch, "(%a+)%s*:(.*)")
		visSearch.mode = key
	else
		visSearch.mode = "title"
	end
	visSearch.value = ToLower(value or CurSearch)
end

local function FetchNextAvailableStyle( StyleContainer, offset )
	m_styleindex = m_styleindex + offset
	if m_styleindex < 1 then m_styleindex = 1 end
	if m_styleindex > #StyleContainer then m_styleindex = #StyleContainer end
end

local function toggleSortingFromType( type )
	-- increase value from the sorting mode selected.
	m_SortingModes[type] = m_SortingModes[type] + 1
	if m_SortingModes[type] > #availablesortingmodes[type] then
		m_SortingModes[type] = 1
	end
end

local performedCode = false
local inPortal = false
local function GetPreferredSelectionForRandomOrPortal( GroupsAndSongs )
	local CurrentGroup = CurGroup or ""
	local randomPool = {}

	local tempGroupIter = ""
	for i,v in ipairs(GroupsAndSongs) do
		if type(v) == "string" then
			tempGroupIter = v
		end
		-- if no selected group, anything goes - otherwise, make sure the item is from selected group
		local validGroup = (CurrentGroup == "") or (tempGroupIter == CurrentGroup)
		local isSong = type(v) == "table"
		if validGroup and isSong then
			randomPool[#randomPool+1] = v
		end
	end

	if #randomPool == 0 then
		return GroupsAndSongs[CurSong]
	end

	return randomPool[math.random( #randomPool )]
end

local function GetSelectedSong( GroupsAndSongs )
	if GroupsAndSongs[CurSong] == "-RANDOM-" then
		-- Full stop.
		return "-RANDOM-"
	end
	if GroupsAndSongs[CurSong] == "-PORTAL-" then
		if (CurGroup and CurGroup ~= "") then
			return GetPreferredSelectionForRandomOrPortal( GroupsAndSongs )
		end
		return Songs[ math.random(#Songs) ]
	end
	return GroupsAndSongs[CurSong]
end

local StyleTEMP

-- if not m_ChooserDiffs then m_ChooserDiffs = {} end
-- We define the curent song if no song is selected
-- We'll offset this value to 3 to avoid problems with Portal and Random not being initialized
-- properly.
if not CurSong then CurSong = 3 end

-- We define the current group to be empty if no group is defined.
if not CurGroup then GurGroup = "" end

-- The player joined.
local Joined = {}

-- The current search
if not CurSearch then CurSearch = "" end

local KeepSearch = true

for pn in ivalues(PlayerNumber) do
	Joined[pn] = GAMESTATE:IsPlayerEnabled(pn)
end

-- Position on the difficulty select that shows up after we picked a song.
local DiffPos = {[PLAYER_1] = 1,[PLAYER_2] = 1}
local LastDiff = {[PLAYER_1] = nil,[PLAYER_2] = nil}

local ColorBasedOnAward = LoadModule("Gameplay/ColorBasedOnAward.lua")

-- The increase offset for when we move with postive.
local IncOffset = 1

-- The decrease offset for when we move with negative.
local DecOffset = 13

-- The center offset of the wheel.
local XOffset = 7

local getJoinedPlayers = function()
	local total = 0
	for k,v in pairs(Joined) do
		if Joined[k] then
			total = total + 1
		end
	end
	return total
end

local getFirstAvailablePlayer = function()
	for k,v in pairs(Joined) do
		if Joined[k] then
			return k
		end
	end
end

local lastSong
local function GetGroupNameBasedOnSort( songToUse )
	local song = type(songToUse) == "table" and songToUse or lastSong[1]
	local modes = {
		group = function() return song:GetGroupName() end,
		artist = function() return ToLower(song:GetDisplayArtist()):sub(1,1) end,
		title = function()
			-- Special case: When toggling this mode, we need to automatically set the
			-- group to the appropiate one.
			local title = ToLower(song:GetDisplayMainTitle()):sub(1,1)
			-- Check it's type.
			if title:match("%W") then
				-- Ok, it's not alphanumerical, so see if it's a number, or it'll go to the other container.
				title = tonumber(title) and "number" or "other"
			end

			return title
		end
	}
	return modes[ getSortingModeFromType("Group") ]()
end

-- This is used for the touch integration, to see if whatever the player has scrolled through has at least hit one song
-- from its origin point.
local hasHitAnotherSong = false
local mainActorFrame = nil

-- If we're in a aspect ratio somewhat about < 16:10, then we need to start resizing.
local bannerXPos = IsVerticalScreen and 0 or (aspectRatio < 1.6 and -160 or -192)
local bannerYPos = IsVerticalScreen and -SCREEN_HEIGHT*.25 or -120

local newsong
local function UpdateCurrentlyPlayingMusic(self,Songs)
	-- Check if its a song.
	if newsong and type(newsong) ~= "string" then
		-- Play Current selected Song Music.
		-- If there's already an audio preview playing on the sound manager, don't perform the change.
		if newsong[1]:GetMusicPath() ~= SOUND:GetMusicPath() then
			SOUND:StopMusic()
			self:stoptweening():sleep(0.1):queuecommand("PlayCurrentSong")
		end
	else
		-- Group / nil, stop music.
		SOUND:StopMusic()
	end
end

local DiffTextFields = {}

local function FetchStepsForWheelItem(pn)
	if inPortal then
		return newsong[DiffPos[pn]+1]
	end
	return GroupsAndSongs[CurSong][DiffPos[pn]+1]
end

-- Change the cursor of Player on the difficulty selector.
local function MoveDifficulty(player,offset)
	if doneWithInput then return end

	-- check if player is joined.
	if not Joined[player] then return end
	if PlayerPage[player].MenuOpen then
		PlayerPage[player].OptionListHandler:MoveSelection(offset,player)
		return
	end
	
	-- Is the current selected item a song?
	if type(newsong) == "string" then return end

	-- Move cursor.
	DiffPos[player] = DiffPos[player] + offset

	-- Keep within boundaries.
	if DiffPos[player] < 1 then DiffPos[player] = 1 end
	if DiffPos[player] > #newsong-1 then DiffPos[player] = #newsong-1 end

	LastDiff[player] = FetchStepsForWheelItem(player):GetDifficulty()

	-- If BothAtOnce is enabled, then it's likely the user is going to play on both areas
	-- at the same time, so match the diffs.
	if tobool(PREFSMAN:GetPreference("BothAtOnce")) then
		for pn in ivalues(PlayerNumber) do
			DiffPos[pn] = DiffPos[player]
			LastDiff[pn] = LastDiff[player]
			MESSAGEMAN:Broadcast("PlayerSwitchedStep",{ Player = pn, Index = DiffPos[pn]+1, Song = newsong })
			MESSAGEMAN:Broadcast("UpdateNotefield",{ pn = pn, Steps = FetchStepsForWheelItem(pn), Style = StyleTEMP[m_styleindex] })
			MESSAGEMAN:Broadcast("UpdateOnlineChartInfo",{ Player = pn, Steps = FetchStepsForWheelItem(pn), Toggle = PlayerPage[pn].Index == 2 })
		end
		return
	end

	-- Send information to other actors so they can use it.
	MESSAGEMAN:Broadcast("PlayerSwitchedStep",{ Player = player, Index = DiffPos[player]+1, Song = newsong })
	MESSAGEMAN:Broadcast("UpdateNotefield",{ pn = player, Steps = FetchStepsForWheelItem(player), Style = StyleTEMP[m_styleindex] })
	MESSAGEMAN:Broadcast("UpdateOnlineChartInfo",{ Player = player, Steps = FetchStepsForWheelItem(player), Toggle = PlayerPage[player].Index == 2 })
end

local wheelItemYSpacing = 45
local wheelItemCenterSpacing = 25
local function CalcOffsetForSpot( valoffset, i )
	local yoffset = valoffset*wheelItemYSpacing
	-- Off from center.
	local needsTweening = true
	if valoffset ~= 0 then
		-- Calculate the points after the wheel.
		if valoffset > 0 then
			yoffset = wheelItemCenterSpacing+valoffset*wheelItemYSpacing
			if (i-XOffset) >= 6 then
				needsTweening = false
				yoffset = (wheelItemYSpacing * -13) + (-wheelItemCenterSpacing+valoffset*wheelItemYSpacing)
			end
		end
		if valoffset < 0 then
			yoffset = -wheelItemCenterSpacing+valoffset*wheelItemYSpacing
			if (i-XOffset) <= -7 then
				needsTweening = false
				yoffset = (wheelItemYSpacing * 14) + (-wheelItemCenterSpacing+valoffset*wheelItemYSpacing)
			end
		end
	end

	return yoffset
end

-- Move the wheel, We define the Offset using +1 or -1.
-- We parse the Songs also so we can get the amount of songs.
local function MoveSelection(self,offset,Songs,IsTouch)
	if doneWithInput then return end

	-- check if player is joined if the parameter is given.
	if self.pn then
		if not Joined[self.pn] and not IsTouch then self.pn = nil return end
		if PlayerPage[self.pn].MenuOpen then self.pn = nil return end
	end

	-- Curent Song + Offset.
	CurSong = CurSong + offset
	
	-- Check if curent song is further than Songs if so, reset to 1.
	if CurSong > #Songs then CurSong = 1 end
	-- Check if curent song is lower than 1 if so, grab last song.
	if CurSong < 1 then CurSong = #Songs end
	
	if (offset ~= 0 or IsTouch) then
		newsong = GetSelectedSong(Songs)
		if type(newsong) == "table" then
			lastSong = newsong
		end
	end
	
	-- Simple flag to avoid sending SongInfoBox repeat data.
	local songChanged = newsong and GAMESTATE:GetCurrentSong() ~= newsong[1] or true
	if newsong then
		GAMESTATE:SetCurrentSong(newsong[1])
	end
	-- Set the offsets for increase and decrease.
	DecOffset = DecOffset + offset
	IncOffset = IncOffset + offset

	if DecOffset > 13 then DecOffset = 1 end
	if IncOffset > 13 then IncOffset = 1 end

	if DecOffset < 1 then DecOffset = 13 end
	if IncOffset < 1 then IncOffset = 13 end
	
	-- Set the offset for the center of the wheel.	
	XOffset = XOffset + offset
	if XOffset > 13 then XOffset = 1 end
	if XOffset < 1 then XOffset = 13 end

	if Songs[CurSong] == "-PORTAL-" then
		inPortal = XOffset
	end

	-- If we are calling this command with an offset that is not 0 then do stuff.
	if offset ~= 0 then

		-- self:GetChild("Highlight"):GetChild("Play"):playcommand("TweenAnimation",{offset=offset})

		-- For every part on the wheel do.
		for i = 1,13 do
			-- Calculate current position based on song with a value to get center.
			local pos = CurSong+(6*offset)
		
			-- Keep it within reasonable values.
			while pos > #Songs do pos = pos-#Songs end
			while pos < 1 do pos = #Songs+pos end

			local offsetval = CalcOffsetForSpot( (i-XOffset), i )
			local prevOffset = CalcOffsetForSpot( (i-XOffset)+offset, i )

			self:GetChild("Wheel"):GetChild("Container"..i):stoptweening(0.1):y( prevOffset ):linear( IsTouch and 0 or 0.08 ):y( offsetval )
			:diffusealpha( offsetval > -230 and 1 or 0 )


			-- Here we define what the wheel does if it is outside the values.
			-- So that when a part is at the bottom it will move to the top.
			if (i == IncOffset and offset == -1) or (i == DecOffset and offset == 1) then
				-- Send the current information back to the wheel to update its contents.
				self:GetChild("Wheel"):playcommand("UpdateContainer",{Data=Songs[pos],Offset = i})
			end
		end
	
		self:GetChild("SongInfoBox"):playcommand("UpdateSongInfo",{Data=newsong,visSearch=visSearch})
		self:GetChild("Toolkit"):GetChild("GroupChooser"):playcommand("UpdateSongInfo",{Data=newsong})
		-- We have a top banner and an under banner to make smooth transisions between songs.

		-- Check if it's a song.
		if type(Songs[CurSong]) ~= "string" then
			-- It is a song, so we load the under banner.
			-- self:GetChild("ModeButtons"):playcommand("UpdateAvailableSteps",{Data=Songs[CurSong]})
		end

	-- We are on an offset of 0.
	else

		if songChanged then
			self:GetChild("SongInfoBox"):playcommand("UpdateSongInfo",{Data=newsong,visSearch=visSearch})
			self:GetChild("Toolkit"):GetChild("GroupChooser"):playcommand("UpdateSongInfo",{Data=newsong})
		end

		-- For every part of the wheel do.
		for i = 1,13 do	

			-- Offset for the wheel items.
			local off = i + XOffset

			-- Stay withing limits.
			while off > 13 do off = off-13 end
			while off < 1 do off = off+13 end

			-- Get center position.
			local pos = CurSong+i

			-- If item is above 6 then we do a +13 to fix the display.
			if i > 6 then
				pos = CurSong+i-13
			end

			-- Keep pos withing limits.
			while pos > #Songs do pos = pos-#Songs end
			while pos < 1 do pos = #Songs+pos end

			-- Send the current information back to the wheel to update its contents.
			self:GetChild("Wheel"):playcommand("UpdateContainer",{Data=Songs[pos],Offset = off})
		end

		self:GetChild("SongInfoBox"):playcommand("UpdateSongInfo",{Data=newsong})
		self:GetChild("Toolkit"):GetChild("GroupChooser"):playcommand("UpdateSongInfo",{Data=newsong})

		-- self:GetChild("ModeButtons"):playcommand("UpdateAvailableSteps",{Data=newsong})
	end
	
	-- Check if it's a song.
	for pn in ivalues(PlayerNumber) do 
		if GAMESTATE:IsPlayerEnabled(pn) then
			if type(newsong) ~= "string" then
				local foundDiff = false

				for i=2,#newsong do
					if newsong[i]:GetDifficulty() == LastDiff[pn] then
						DiffPos[pn] = i-1
						foundDiff = true
						break
					end
				end
				if not foundDiff then
					if DiffPos[pn] > #newsong-1 then
						DiffPos[pn] = #newsong-1
					end
				end

				
				MESSAGEMAN:Broadcast("PlayerSwitchedStep",{ Player = pn, Force = true, Index = DiffPos[pn]+1, Song = newsong })
				-- Do NOT update the leaderboard if the user is using the touchscreen/mouse to drag around, or there will be
				-- MASSIVE lag.
				if not IsTouch then
					MESSAGEMAN:Broadcast("UpdateOnlineChartInfo",{ Player = pn, Steps = FetchStepsForWheelItem(pn), Toggle = PlayerPage[pn].Index == 2 })
				end
				
				MESSAGEMAN:Broadcast("UpdateNotefield",{ pn = pn, Song = newsong[1], Steps = FetchStepsForWheelItem(pn), Style = StyleTEMP[m_styleindex] })
			else
				MESSAGEMAN:Broadcast("PlayerSwitchedStep",{ Player = pn, Song = "" })
			end
		end
	end

	if Songs[CurSong] == "-PORTAL-" then
		self:GetChild("Wheel"):playcommand("UpdateContainer",{Data=newsong,Offset = XOffset})
		-- self:GetChild("ModeButtons"):playcommand("UpdateAvailableSteps",{Data=newsong})
	else
		if inPortal then
			-- find the wheel that had the portal item.
			self:GetChild("Wheel"):playcommand("UpdateContainer",{Data="-PORTAL-",Offset = inPortal})
			inPortal = false
		end
	end

	-- Check if offset is not 0.
	if offset ~= 0 then
		
		-- self:GetChild("Slider"):stoptweening():linear(.1):y( SCREEN_HEIGHT * (CurSong/#Songs) )

		-- Stop all the music playing, Which is the Song Music
		if not IsTouch then
			UpdateCurrentlyPlayingMusic(self,Songs)
		end

		self:GetChild("ChangeSound"):play()
	else
		-- self:GetChild("Slider"):y( SCREEN_HEIGHT*(CurSong/#Songs) )

		if IsTouch and hasHitAnotherSong then
			UpdateCurrentlyPlayingMusic(self,Songs)
		end
	end

	-- We're done, clear the player buffer.
	self.pn = nil
end

-- If online is available, process the information the currently available matches to fetch their charts.
local function FetchOnlineInfoCharts( GroupsAndSongs )
	if not NETMAN or not NETMAN:IsConnectionEstablished() then return end
	if (CurGroup == LastGroup or CurGroup == "") and not inSearchMode then return end
	LastGroup = CurGroup
	
	local charts = {}
	for index,wheelItem in ipairs( GroupsAndSongs ) do
		if type(wheelItem) == "table" and #wheelItem > 1 then
			for diff = 2,#wheelItem do
				charts[#charts+1] = wheelItem[diff]:GetChartKey()
			end
		end
	end

	GAMESTATE:Env()["OnlineChartKeyInfo"] = charts
end

local function CheckStageAvailability( song )
	if not song then return false end
	return isEventMode or (song:GetStageCost() <= GAMESTATE:GetNumStagesLeft(GAMESTATE:GetMasterPlayerNumber()))
end

local WheelItemsActorFrame
local WIAF = Def.ActorFrame{
	InitCommand=function(self)
		WheelItemsActorFrame = self
		-- Hide the actorframe, the Actor Proxy will deal with showing it.
		self:visible(false)
	end,
	CancelCommand=function(self)
		self:RunCommandsRecursively(function(self)
			self:easeinexpo(0.2):diffusealpha(0)
		end)
	end,

	-- Generate actors that will represent the background items for the wheel.
	-- To avoid generating a lot of sprite instances, we'll use actor proxies to point to these instead.
	LoadActor(THEME:GetPathB("_frame","3x1"), "Wheel GroupClosed", WheelItemWidth - 70)..{ Name="GroupCollapse" },
	LoadActor(THEME:GetPathB("_frame","3x1"), "Wheel GroupOpen", WheelItemWidth - 70)..{ Name="GroupExpand" },
	LoadActor(THEME:GetPathB("_frame","3x1"), "Wheel Item", WheelItemWidth - 70)..{ Name="ItemBG" },
	LoadActor(THEME:GetPathB("_frame","3x1"), "Wheel Special Random", WheelItemWidth - 70)..{ Name="ItemSpecialRandom" },
	LoadActor(THEME:GetPathB("_frame","3x1"), "Wheel Special Portal", WheelItemWidth - 70)..{ Name="ItemSpecialPortal" }
}

-- This is the main function, Its the function that contains the wheel.
return function(Style)

	StyleTEMP = Style

	-- Load the songs from the Songs.Loader module.
	if m_styleindex > #Style then m_styleindex = 1 end
	Songs = ModuleUtils.SongLoader(Style[m_styleindex])

	-- No songs were found on the current mode? Check the next one.
	while( #Songs == 0 ) do
		m_styleindex = m_styleindex + 1
		-- We're out of bounds, stop operation.
		if m_styleindex > #Style then
			m_styleindex = #Style
			break
		end

		Songs = ModuleUtils.SongLoader(Style[m_styleindex])
	end

	if (KeepSearch and CurSearch ~= "") then
		inSearchMode = true
		fetchSearchModeFromFlag()
	end

	local initialFetchForPlayerProfileSong = false

	if not GAMESTATE:Env()["GroupsAndSongs"] then
		-- When entering this screen for the first time, the selection is going to be null, so make the player be pointed to
		-- their last played song just like in the engine wheel.
		local songToFind = nil
		local pMatch = PROFILEMAN:GetProfile( GAMESTATE:GetMasterPlayerNumber() )

		if pMatch then
			songToFind = pMatch:GetLastPlayedSong()
			if songToFind then
				initialFetchForPlayerProfileSong = songToFind
				CurGroup = songToFind:GetGroupName()
			end
		end

		GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,CurGroup,KeepSearch and CurSearch or nil,getSortingModeFromType("Group"),getSortingModeFromType("Song"))
	end

	GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]

	-- Sort the Songs and Group.
	-- local GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,CurGroup,KeepSearch and CurSearch or nil,getSortingModeFromType("Group"),getSortingModeFromType("Song"))

	newsong = GroupsAndSongs[CurSong]

	local function FindNextAvailableSongInGroup()
		local attempts = 0
		local num = 0
		while ( attempts < 13 ) do
			if type(GroupsAndSongs[num]) == "table" then
				CurSong = num
				return true
			end
			num = num + 1
			attempts = attempts + 1
		end
		return false
	end
	
	-- Set CurSong to the right group.
	local function FindSongInGroup(song)
		for i,v in ipairs(GroupsAndSongs) do
			if type(v) == "table" and v[1] == song then
				CurSong = i
				return true
			end
		end

		return false
	end

	-- Needed for the initial state-player profile fetching.
	if initialFetchForPlayerProfileSong then
		FindSongInGroup( initialFetchForPlayerProfileSong )
		newsong = GroupsAndSongs[CurSong]
	end
	
	-- The actual wheel.
	local Wheel = Def.ActorFrame{
		Name="Wheel",
		InitCommand=function(self)
			self.ypos = 0
			local xpos = SCREEN_WIDTH * (aspectRatio < 1.34 and .16 or .15)
			if IsVerticalScreen then
				xpos = 0
				self.ypos = SCREEN_HEIGHT*.15
				self:zoom(0.7)
			end
			self:xy( xpos,self.ypos )
			self:AddWrapperState()
		end,
		CloseMenuCommand=function(self)
			self:diffuse(Color.White)
		end,
		UpdateContainerCommand=function(self,params)
			-- It's a song, Display song title.
			local container = self:GetChild("Container"..params.Offset)
			-- This is a shortcut to obtain the children actors to modify.
			local c = container:GetChildren()

			local isSong = type(params.Data) == "table"
			c.Title:visible(isSong)
			c.GroupName:visible(not isSong)
			c.WheelPointer:SetTarget( WheelItemsActorFrame:GetChild( not isSong and "GroupCollapse" or "ItemBG" ) )
			if isSong then
				c.Title:settext(params.Data[1]:GetDisplayMainTitle(),params.Data[1]:GetTranslitMainTitle()):diffuse(color("#C6F8FE"))
				c.SubTitle:diffuse(color("#C6F8FE"))
				c.JacketContainer:visible( params.Data[1]:HasJacket() )
				if c.JacketContainer:GetVisible() and showJackets and params.Data[1]:GetJacketPath() then
					-- Load the cached version of the jacket instead.
					-- TODO: Maybe add support to cache jackets.
					c.JacketContainer:LoadFromCached("jacket",params.Data[1]:GetJacketPath() ):setsize( c.JacketContainer.expectedSize, c.JacketContainer.expectedSize )

					-- c.JacketContainer:Load( params.Data[1]:GetJacketPath() ):setsize( c.JacketContainer.expectedSize, c.JacketContainer.expectedSize )
				end

				-- Does the player have enough stages to play this song?
				if not CheckStageAvailability(params.Data[1]) then
					-- TODO: Redo this color by fetching it to the ItemBG item itself.
					c.Title:diffuse(color("#777777"))
					c.SubTitle:diffuse(color("#777777"))
				end
			else
				-- It is not a song, Display group name instead.
				c.GroupName:settext(params.Data)

				for i,player in ipairs( PlayerNumber ) do
					local prof = PROFILEMAN:GetProfile(player)
					if params.Data == prof:GetDisplayName() then
						c.GroupName:diffuse( PlayerColor(player) )
					end
				end

				c.JacketContainer:visible( false )

				if CurGroup == params.Data then
					c.WheelPointer:SetTarget( WheelItemsActorFrame:GetChild("GroupExpand") )
				end

				if params.Data == "-PORTAL-" then
					c.WheelPointer:SetTarget( WheelItemsActorFrame:GetChild("ItemSpecialPortal") )
				end
				if params.Data == "-RANDOM-" then
					c.WheelPointer:SetTarget( WheelItemsActorFrame:GetChild("ItemSpecialRandom") )
				end
			end

			c.Title:zoom(.8):y(0)
			c.SubTitle:settext("")

			-- Check if it's a song.
			if type(params.Data) ~= "string" then
				-- Check if song has subtitle.
				if params.Data[1]:GetDisplaySubTitle() ~= "" then 
					-- It does have a subtitle so resize the title to fit it.
					c.Title:zoom(.75):y(-6)
				end
				-- Set subtitle and artist to the values it has.
				c.SubTitle:settext(params.Data[1]:GetDisplaySubTitle())

				-- Highlight the part of the text that was found in the search.
				if inSearchMode then
					local states = TF_WHEEL.SearchStates
					local touse = {
						["Title"] = params.Data[1]:GetDisplayMainTitle(),
						["SubTitle"] = params.Data[1]:GetDisplaySubTitle(),
					}
					-- Highlight the part of the text that was found in the search.
					for i,v in pairs( touse ) do
						-- Does the user want this search?
						if states[ToLower(i)] then
							local matches, start = string.find( ToLower(v), visSearch.value )
							local searchlength = string.len(visSearch.value)
							-- We only need to do this if we have any kind of match.
							if matches then
								container:GetChild(i):AddAttribute( matches-1, { Length = searchlength, Diffuse = GameColor.Custom["MenuButtonBorder"] } )
							end
						end
					end
				end
			end
		end
	}

	local function UpdateSortingFolders()
		mainActorFrame:GetChild("Toolkit"):GetChild("SortingMethodsInsideFolder"):playcommand("UpdateSongSort",{inSearchMode = inSearchMode, Mode = getSortingModeFromType("Song")})
		mainActorFrame:GetChild("Toolkit"):GetChild("SortingMethodsFolders"):playcommand("UpdateGroupSort",{inSearchMode = inSearchMode, Mode = getSortingModeFromType("Group")})
	end

	--[[
	local ModeButtons = Def.ActorFrame{
		Name="ModeButtons",

		InitCommand=function(self)
			local margin = needsSDVersion and 310 or 329
			local zoomset = needsSDVersion and 1.8 or 1
			self:GetChild("Next"):x(margin):zoom( zoomset )
			self:GetChild("IconNext"):x((margin-11)):zoom(0.7)
			
			self:GetChild("Prev"):x(-margin):zoom(zoomset):zoomx(-zoomset)
			self:GetChild("IconPrev"):x(-(margin-11)):zoom(0.7)

			self:GetChild("ModeBG"):zoom( 1.1 )
			self:GetChild("ModeBG"):GetChild("ModeIcon"):x(-166):zoom(0.7)
			self:GetChild("ModeBG"):GetChild("CurMode"):x(-110):uppercase(true):halign(0):zoom(1.5):diffuse(color("#9EEAEA"))
		end,

		NextModeCommand=function(self,params)
			self:GetChild("ModeBG"):stoptweening():x(20):glow(color("#ffffff55")):decelerate(0.2):x(0):glow(color("#ffffff00"))
		end,
		PrevModeCommand=function(self,params)
			self:GetChild("ModeBG"):stoptweening():x(-20):glow(color("#ffffff55")):decelerate(0.2):x(0):glow(color("#ffffff00"))
		end,

		UpdateAvailableStepsCommand=function(self,params)
			-- Update current text with the mode.
			if GroupsAndSongs[CurSong] and GroupsAndSongs[CurSong][DiffPos[PLAYER_1]+1] then
				local stepstype = ToEnumShortString(GroupsAndSongs[CurSong][DiffPos[PLAYER_1]+1]:GetStepsType())
				self:GetChild("ModeBG"):GetChild("ModeIcon"):Load( THEME:GetPathG("","_StepsVector/".. stepstype..".svg" ) )
				self:GetChild("ModeBG"):GetChild("CurMode"):settext( THEME:GetString("StepsType",stepstype ) )

				self:GetChild("IconPrev"):Load( Style[m_styleindex-1] and THEME:GetPathG("","_StepsVector/"..Style[m_styleindex-1]..".svg" or nil ) )
				self:GetChild("IconNext"):Load( Style[m_styleindex+1] and THEME:GetPathG("","_StepsVector/"..Style[m_styleindex+1]..".svg" or nil ) )
			end
		end,

		Def.Sprite{ Name="Prev", Texture=THEME:GetPathG("","SelectMusic/".. (needsSDVersion and "Small" or "") .."ModeSwitchButton") },
		Def.Sprite{ Name="IconPrev" },
		Def.Sprite{ Name="Next", Texture=THEME:GetPathG("","SelectMusic/".. (needsSDVersion and "Small" or "") .."ModeSwitchButton") },
		Def.Sprite{ Name="IconNext" },
		Def.ActorFrame{
			Name="ModeBG",
			Def.Sprite{ Texture=THEME:GetPathG("","SelectMusic/CurrentMode") },
			Def.Sprite{ Name="ModeIcon" },
			-- Actual information
			Def.BitmapText{ Font="_Bold", Name="CurMode" }
		}
	}
	]]
		
	-- For every item on the wheel do.
	for i = 1,13 do
		-- Grab center of wheel.
		local offset = i - 7
		
		-- Also grab center of wheel.
		local pos = CurSong+i-7

		-- But we keep it within limits.
		while pos > #GroupsAndSongs do pos = pos-#GroupsAndSongs end
		while pos < 1 do pos = #GroupsAndSongs+pos end
		
		-- Append to the wheel.
		Wheel[#Wheel+1] = Def.ActorFrame{
			Name="Container"..i,

			-- Set position of item.
			OnCommand=function(self)
				self:y( CalcOffsetForSpot( (i-XOffset), i ) )
				self:diffusealpha(0):linear(0.5):diffusealpha(1)
			end,

			CancelCommand=function(self)
				self:easeoutexpo(0.5):diffusealpha(0)
			end,

			UpdateSizeCommand=function(self)
				self:GetChild("GroupName"):stoptweening():easeoutexpo(0.2):x( -WheelItemWidth*.5 + 4 ):maxwidth(WheelItemWidth)
				self:GetChild("Title"):stoptweening():easeoutexpo(0.2):x( -WheelItemWidth*.5 + 4 ):maxwidth(WheelItemWidth)
				self:GetChild("SubTitle"):stoptweening():easeoutexpo(0.2):x( -WheelItemWidth*.5 + 4 ):maxwidth(WheelItemWidth)
				self:GetChild("JacketContainer"):stoptweening():easeoutexpo(0.2):x( WheelItemWidth*.5 - 14 )
			end,

			Def.ActorProxy{
				Name="WheelPointer",
				OnCommand=function(self)
					local item = "ItemBG"
					-- Check if we are on group.
					if type(GroupsAndSongs[pos]) == "string" then
						item = CurGroup == GroupsAndSongs[pos] and "GroupExpand" or "GroupCollapse"
						if GroupsAndSongs[pos] == "-PORTAL-" then
							item = "ItemSpecialPortal"
						end
						if GroupsAndSongs[pos] == "-RANDOM-" then
							item = "ItemSpecialRandom"
						end
					end
					self:SetTarget( WheelItemsActorFrame:GetChild(item) )
				end
			},

			-- Jacket image spot (if available)
			Def.Sprite{
				Name="JacketContainer",
				InitCommand=function(self)
					self.expectedSize = 40
					self:x( WheelItemWidth*.5 - 14 )
				end
			},
			

			-- Title for group names.
			Def.BitmapText{
				Name="GroupName",
				Font="_kurinto sans bold/20px",
				OnCommand=function(self) 
					-- Check if we are on group.
					self:maxwidth( WheelItemWidth )
					if type(GroupsAndSongs[pos]) == "string" then
						-- Show group name.
						self:settext(GroupsAndSongs[pos])
					end

					-- Set the size of the text and the location.
					self:halign(0):x( -WheelItemWidth*.5 + 4 ):diffuse( color("#C6F8FE") )

					-- The check above failed, so it must be a song (or group) without subtitle.
					:zoom(.8)
				end
			},

			-- Song Title for on wheel.
			Def.BitmapText{
				Name="Title",
				Font="_Kurinto Medium",
				OnCommand=function(self) 
					-- Check if we are on group.
					self:maxwidth( WheelItemWidth - 8 )

					-- Set the size of the text and the location.
					self:halign(0):xy( -WheelItemWidth*.5 + 4 ,-6):diffuse( color("#C6F8FE") )

					if not CheckStageAvailability(GroupsAndSongs[pos][1]) then
						self:diffuse(color("#777777"))
					end 

					-- Check if it's a song.
					if type(GroupsAndSongs[pos]) == "table" then
						self:settext(GroupsAndSongs[pos][1]:GetDisplayMainTitle(),GroupsAndSongs[pos][1]:GetTranslitMainTitle())
						-- Check if song subtitle is empty.
						if GroupsAndSongs[pos][1]:GetDisplaySubTitle() ~= "" then 
							self:zoom(.75)
							return
						end
					end

					-- The check above failed, so it must be a song (or group) without subtitle.
					self:y(0):zoom(.8)
				end
			},

			-- The subtitle.
			Def.BitmapText{
				Name="SubTitle",
				Font="_kurinto sans med/20px",
				OnCommand=function(self)
					-- Check if we are on group.
					if type(GroupsAndSongs[pos]) ~= "string" then
						-- Set Subtitle.
						self:settext(GroupsAndSongs[pos][1]:GetDisplaySubTitle())
					end
					
					-- Set size and colour.
					self:zoom(.5):xy(-WheelItemWidth*.5 + 4,8):halign(0):maxwidth(650):skewx(-0.2)
				end
			}
		}	
	end
	
	-- touch centric values
	local mouseIsInArea = false
	local mouseIsInDragMode = false
	local lastYPosMouse = 0

	RestoreWheelFromSearch = function()
		if newsong == nil or lastSong == nil then return end

		local cursong = type(newsong) == "string" and lastSong[1] or newsong[1]
		CurGroup = GetGroupNameBasedOnSort(cursong)
		-- Reset the groups location so we dont bug.
		GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,"")
		GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]
		-- Set CurSong to the right group.
		for i,v in ipairs(GroupsAndSongs) do
			if v == CurGroup then
				CurSong = i
			end
		end						
		-- Set the current group.
		GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,CurGroup,CurSearch,getSortingModeFromType("Group"),getSortingModeFromType("Song"))
		GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]
		FindSongInGroup( cursong )
	end

	BeginSearch = function()
		enteredSearch = true
		mainActorFrame:GetChild("Toolkit"):GetChild("SearchBar"):playcommand("BeginSearch")
		mainActorFrame:playcommand("DimMainItems")
		SCREENMAN:AddNewScreenToTop("SearchSongWide")
		local searchSettings = {
			Question = "",
			InitialAnswer = CurSearch or "",
			MaxInputLength = 255,
			OnOK = function(answer)
				CurSearch = answer
				mainActorFrame:GetChild("Toolkit"):playcommand("SearchPerformed",{ String = CurSearch })
				mainActorFrame:playcommand("RestoreItemDiffuse")
				if answer == "" then
					-- Reset all states from the search.
					for k,v in pairs( TF_WHEEL.SearchStates ) do TF_WHEEL.SearchStates[k] = false end
					RestoreWheelFromSearch()
					inSearchMode = false
					UpdateSortingFolders()
					mainActorFrame:GetChild("Wheel"):playcommand("UpdateContainer",{Data=Songs[CurSong],Offset = XOffset})
					MoveSelection(mainActorFrame,0,GroupsAndSongs,true)
					return
				end
				local tempGroups = {}
				local matches = 0
				tempGroups, matches = ModuleUtils.GroupSet(Songs,CurGroup,CurSearch,getSortingModeFromType("Group"),getSortingModeFromType("Song"))
				-- Update all wheel entries to have the result objects.
				inSearchMode = answer ~= ""
				UpdateSortingFolders()
				if matches > 0 then
					fetchSearchModeFromFlag()
					GAMESTATE:Env()["GroupsAndSongs"] = tempGroups
					GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]
					hasHitAnotherSong = true
					-- Force the new song to become the first result on the list.
					newsong = GroupsAndSongs[1]
					lastSong = newsong
				else
					-- Reset all states from the search.
					for k,v in pairs( TF_WHEEL.SearchStates ) do TF_WHEEL.SearchStates[k] = false end
					inSearchMode = false
					MESSAGEMAN:Broadcast("SystemNotification",{Text="No results.", Icon=THEME:GetPathG("NotificationIcon","error"), Pos={SCREEN_CENTER_X, 80}})
					CurSearch = ""
					mainActorFrame:GetChild("Toolkit"):GetChild("SearchBar"):playcommand("SearchPerformed",{ String = CurSearch })
					RestoreWheelFromSearch()
					UpdateSortingFolders()
					mainActorFrame:GetChild("Wheel"):playcommand("UpdateContainer",{Data=Songs[CurSong],Offset = XOffset})
				end
				enteredSearch = false
				MoveSelection(mainActorFrame,0,GroupsAndSongs,true)
				-- FetchOnlineInfoCharts(GroupsAndSongs)
			end,
			OnCancel = function()
				-- mainActorFrame:GetChild("SearchBar"):GetChild("Text"):settext( CurSearch ~= "" and CurSearch or SearchSongLabel )
				if inSearchMode then
					-- Reset all states from the search.
					for k,v in pairs( TF_WHEEL.SearchStates ) do TF_WHEEL.SearchStates[k] = false end
					inSearchMode = false
					RestoreWheelFromSearch()
					UpdateSortingFolders()
					MoveSelection(mainActorFrame,0,GroupsAndSongs,true)
				else
					mainActorFrame:GetChild("Toolkit"):playcommand("SearchPerformed",{ String = CurSearch })
				end
				enteredSearch = false
				mainActorFrame:playcommand("RestoreItemDiffuse")
			end
		};
		SCREENMAN:GetTopScreen():Load(searchSettings)
	end

	local PlayerInfoPanes = Def.ActorFrame{Name="PlayerInfoPanes"}

	local isOnline = NETMAN ~= nil and NETMAN:IsConnectionEstablished()

	for player in ivalues(PlayerNumber) do
		PlayerPage[player] = {Index=1,MenuOpen=false,OptionListHandler=nil}

		local PrefsManager = LoadModule("Save.PlayerPrefs.lua")
		PrefsManager:Load( CheckIfUserOrMachineProfile(string.sub(player,-1)-1).."/OutFoxPrefs.ini" )

		local LowerAreaBG = LoadModule("UI/UI.ButtonBox.lua")(PlayerAreaWidth,240,3,nil,true)

		local optionlist = LoadModule("UI/UI.OptionList.lua"){
			-- Settings
			InputTiedTo = player,
			NumChoices = 6,
			TranslateValueNames = true,
			ColorBooleanSwitches = true,
			ItemWidth = needsSDVersion and 280,
			UseMetatable = true,
			TransformationCommand = function(self, index)
				self:xy( 0, 42 * index )
			end,
			-- We'll control this optionlist externally.
			UseDedicatedController = true,
			Cursor = function(width, player)
				return Def.Quad{
					OnCommand=function(self)
						self:zoomto( width, 42 ):diffuse(PlayerCompColor(player)):diffusealpha(0.2):fadebottom(1):blend('add')
					end,
					CursorFrameChangeCommand=function(self,params)
						self:zoomtowidth( params.isMenuButton and 46 or params.Width-4 )
						self:diffusealpha( params.isMenuButton and 0.6 or 0.2 ):fadebottom( params.isMenuButton and 0 or 1)
					end
				}
			end,
			Frame = LoadModule("Theme.GenerateThemeFrame.lua"),
			BoolImage = THEME:GetPathG("","check"),
			BackExitsScreen = false,
			ShowScroller = false,
			SelectSendsExitMessage = true,
			-- Cursor = Def.Quad{ InitCommand=function(self) self:zoomto(300,28):diffuse(color("#ffffff55")) end },
			-- Choices to be shown on the list.
			List = LoadModule("Menus/SelectMusicOptions.lua")
		}
		
		local playerInfo = Def.ActorFrame{
			Name=player,
			InitCommand=function(self)
				local spacing = clamp(scale(aspectRatio,1.60,1.77,186,200), 186, 200)
				if needsSDVersion then
					spacing = 160
				end
				self:x( player == PLAYER_1 and spacing or SCREEN_WIDTH-spacing  )
				:visible( GAMESTATE:IsPlayerEnabled(player) )
			end,
			OnCommand=function(self)
				self:diffusealpha(0):easeoutexpo(0.4):diffusealpha(1)
				-- self:playcommand("ToggleNotefield",{Player = player, State = false})
			end,
			OffCommand=function(self)
				self:easeinexpo(0.2):diffusealpha(0)
				PrefsManager:SaveToFile()
			end,
			PlayerSwitchedStepMessageCommand=function(self,params)
				if params.Player ~= player then return end
				if not Joined[player] then return end

				-- Song must be a table, so it contains data.
				if type(params.Song) == "string" then return end

				if params.Index > #params.Song then
					params.Index = #params.Song
				end

				local colboost = BoostColor( ColorDarkTone(GameColor.Difficulty[ params.Song[params.Index]:GetDifficulty() ]), 1.2)
				LowerAreaBG:ChangeColor( colboost, BoostColor(colboost, 0.5) )
			end,
			PlayerJoinedMessageCommand=function(self)
				self:visible(true)
			end,
			ToggleNotefieldCommand=function(self,params)
				if player ~= params.Player then return end

				if not params.SkipNoteField then
					self:GetChild("NoteField"):visible( params.State )
				end

				local closeY = isOnline and 264 or 280
				local closeHeight = isOnline and 84 or 48

				if self:GetChild("ButtonPreview") then
					self:GetChild("ButtonPreview"):stoptweening():easeoutexpo(0.2)
					:y( params.State and 0 or 220 )
				end

				self:GetChild("AreaBG"):stoptweening():easeoutexpo(0.2)
				:y( SCREEN_CENTER_Y+( params.State and 145 or 280)  )
				LowerAreaBG:UpdateSize( PlayerAreaWidth, params.State and 330 or 50, 2, "easeoutexpo", 0.2  )
			end,
			LeavingOptionsMenuMessageCommand=function(self,params) self:playcommand("CloseMenu",params) end,
			OptionListTopOfTreeMessageCommand=function(self,params)
				mainActorFrame:playcommand("RestoreItemDiffuse")
				self:playcommand("CloseMenu",params)
			end,
			BeginSearchMessageCommand=function(self,params) self:playcommand("CloseMenu",params) end,
			ToggleLeaderboardMessageCommand=function(self,params)
				if not isOnline then return end
				if player ~= params.Player then return end

				local newindex = PlayerPage[player].Index == 1 and 2 or 1
				self:GetParent():GetParent():playcommand("ButtonPressed",{Player=player,Index=newindex})
			end,
			CloseMenuCommand=function(self,params)
				if params.Player ~= player then return end
				-- lua.ReportScriptError(player)
				-- The player has backed out of the menu, it's time to close it.
				PlayerPage[player].MenuOpen = false
				self:GetChild("OptionList"):diffusealpha(0)
				self:GetChild("DiffSelector"):visible(true)

				local TooglenumbPoss = {45,-50}
				self:GetChild("OptionMenuToggle"):stoptweening():easeoutexpo(0.2):diffusealpha(1)
				:y( SCREEN_CENTER_Y+( TooglenumbPoss[PlayerPage[player].Index] ) )
			end,
			ButtonPressedCommand=function(self,params)
				if not isOnline then return end
				if player ~= params.Player then return end

				PlayerPage[player].Index = params.Index
				local newop = PlayerPage[player].Index

				self:GetChild("Leaderboard"):visible( newop == 2 )
				:stoptweening():linear(0.2):diffusealpha( newop == 2 and 1 or 0 )
				
				-- self:GetChild("OptionMenuToggle"):visible(newop == 1)
				local TooglenumbPoss = PlayerPage[player].MenuOpen and {42,10} or {42,-50}
				self:GetChild("OptionMenuToggle"):stoptweening():easeoutexpo(0.2)
				:diffusealpha( (newop == 2 and PlayerPage[player].MenuOpen) and 0 or 1 )
				:y( SCREEN_CENTER_Y+( TooglenumbPoss[newop] ) )
				-- self:GetChild("ToggleNotefieldPreview"):visible(newop == 1)
				
				self:playcommand("EnableLeaderboard",{ Player = player, Steps = newsong[DiffPos[player]+1], Toggle = PlayerPage[player].Index == 2  })

				LowerAreaBG:UpdateSize( PlayerAreaWidth, newop == 2 and 330 or 240, 2, "easeoutexpo", 0.2  )
				self:GetChild("AreaBG"):stoptweening():easeoutexpo(0.2)
				:y( SCREEN_CENTER_Y+( newop == 2 and 145 or 190)  )

				-- self:GetChild("ChartInfo").IsLocked = newop == 2
				self:GetChild("ChartInfo"):stoptweening():easeoutexpo(0.2)
				:zoom( newop == 2 and .6 or 1 ):y( SCREEN_CENTER_Y - (newop == 2 and 110 or 60)  )
				:playcommand("TweakInfo",{ShowMinimal=newop == 2})

				if newop == 2 then
					self:GetChild("NoteField"):visible(false)
					-- self:playcommand("ToggleNotefield",{Player=player,State=true,SkipNoteField=true})
				else
					local isNotefieldVisible = PrefsManager:Get("ShowNotefieldPreview",true)
					self:GetChild("NoteField"):visible( newop == 1 )
					:stoptweening():linear(0.2):diffusealpha( newop == 1 and 1 or 0 )
					MESSAGEMAN:Broadcast("UpdateNotefield",{ pn = player, Steps = FetchStepsForWheelItem(player), Style = StyleTEMP[m_styleindex] })
					-- self:playcommand("ToggleNotefield",{Player=player,State=true,SkipNoteField=true})
				end
				MESSAGEMAN:Broadcast("UpdateOnlineChartInfo",{ Player = player, Steps = FetchStepsForWheelItem(player), Toggle = PlayerPage[player].Index == 2 })
			end,
			ShowSpeedsMessageCommand=function(self,params)
				if player ~= params.pn then return end
				
				-- Force show the notefield when this menu is opened.
				self:playcommand("ButtonPressed",{Player=player,Index=1})

				self:GetChild("OptionMenuToggle"):stoptweening():easeoutexpo(0.2)
				:y( SCREEN_CENTER_Y - 30 ):diffusealpha(1)
				LowerAreaBG:UpdateSize( PlayerAreaWidth,300, 2, "easeoutexpo", 0.2  )
				self:GetChild("AreaBG"):stoptweening():easeoutexpo(0.2)
				:y( SCREEN_CENTER_Y+160 )
			end,
			HideSpeedsMessageCommand=function(self,params)
				if player ~= params.pn then return end
				self:GetChild("OptionMenuToggle"):stoptweening():easeoutexpo(0.2)
				:y( SCREEN_CENTER_Y + 45 )
				LowerAreaBG:UpdateSize( PlayerAreaWidth,240, 2, "easeoutexpo", 0.2  )
				self:GetChild("AreaBG"):stoptweening():easeoutexpo(0.2)
				:y( SCREEN_CENTER_Y+190 )
			end,

			LoadModule("Wheel/Objects/PlayerArea.lua"){
				Player = player,
				Width = PlayerAreaWidth,
				isOnline = isOnline,
			},

			LoadModule("Wheel/Objects/DiffSelector.lua"){
				Player = player,
				Width = PlayerAreaWidth,
				DiffMoveFunc = MoveDifficulty
			}..{
				PlayerOpenMenuCommand=function(self,params)
					if params.Player ~= player then return end
					self:visible( not PlayerPage[player].MenuOpen )
				end,
			},

			LowerAreaBG:Create()..{
				Name="AreaBG",
				InitCommand=function(self) self:y(SCREEN_CENTER_Y+190) end,
				PlayerSwitchedStepMessageCommand=function(self,params)
					if params.Player ~= player then return end
					if not Joined[player] then return end
					if type(params.Song) == "string" then
						-- lua.ReportScriptError("not song")
						self:stoptweening():linear(0.1):diffusealpha(0)
						return
					end
					self:stoptweening():linear(0.1):diffusealpha(1)
				end
			},

			-- Generates a graph with the fitness progress for the player.
			--[[
			LoadModule("Wheel/Objects/WorkoutGraph.lua"){
				Width = PlayerAreaWidth + 70,
				Height = 70,
				Player = player
			} .. {
				OnCommand=function(self)
					self:zoom(0.7):y( SCREEN_CENTER_Y - 80 )
				end
			},
			]]

			LoadModule("Wheel/Objects/ChartInfo.lua"){
				Width = PlayerAreaWidth,
				Player = player,
				Height = 160,
			}..{
				InitCommand=function(self)
					self:y( SCREEN_CENTER_Y - 60 )
				end,
				PlayerOpenMenuCommand=function(self,params)
					if params.Player ~= player then return end
					self:visible( not PlayerPage[player].MenuOpen )
				end,
				CloseMenuCommand=function(self,params)
					if params.Player ~= player then return end
					self:visible(true)
				end
			},

			optionlist:Create()..{
				Name = "OptionList",
				InitCommand=function(self)
					self:y( SCREEN_CENTER_Y - 270 ):diffusealpha(0)
					optionlist:LockInput( not GAMESTATE:IsPlayerEnabled(player) )
					PlayerPage[player].OptionListHandler = optionlist
				end,
				PlayerOpenMenuCommand=function(self,params)
					if params.Player ~= player then return end
					self:diffusealpha( PlayerPage[player].MenuOpen and 1 or 0 )
					optionlist:BroadcastCurMenuEnterMessage(player)
					if not optionlist.allowedToMove and GAMESTATE:IsPlayerEnabled(params.Player) then
						optionlist:LockInput( false )
					end
				end,
				CloseMenuCommand=function(self,params)
					if params.Player ~= player then return end
					optionlist:BroadcastCurMenuExitMessage(player)
					mainActorFrame:playcommand("RestoreItemDiffuse")
				end
			},

			LoadModule("Wheel/Objects/Notefield.lua"){
				Player = player,
				Width = PlayerAreaWidth,
				isOnline = isOnline,
			}..{ Name="NoteField" },

			LoadModule("UI/UI.GenerateUIWithButtonAction.lua")(
				{
					Width = 48,
					Height = 38,
					Border = 2,
					Pos = { -PlayerAreaWidth*.5 + 40, SCREEN_CENTER_Y + 282 },
					Action = function( self )
						local indx = PlayerPage[player].Index
						local newindex = PlayerPage[player].Index == 1 and 2 or 1
						self:GetParent():GetParent():playcommand("ButtonPressed",{Player=player,Index=newindex})
						self:GetParent():playcommand("ToggleButton",{State=newindex == 2})
					end,
					AddActors = Def.ActorFrame{
						InitCommand=function(self)
							-- By default, the preview area is enabled first.
							self:GetChild("Small"):visible(false)
						end,
						ToggleButtonCommand=function(self,param)
							self:GetChild("Small"):visible(param.State)
							self:GetChild("Online"):visible(not param.State)
						end,
						Def.Sprite{
							Name="Small",
							Texture=THEME:GetPathG("PlayMode","Normal"),
							InitCommand=function(self)
								self:zoom(0.18):rotationz(180)
							end
						},
						Def.Sprite{
							Name="Online",
							Texture=THEME:GetPathG("NotificationIcon","network"),
							InitCommand=function(self)
								self:zoom(0.6)
							end
						},
					}
				}
			)..{ 
				Condition=isOnline,
				Name="ButtonPreview",
				PlayerSwitchedStepMessageCommand=function(self,params)
					if params.Player ~= player then return end
					if not Joined[player] then return end
					if type(params.Song) == "string" then
						-- lua.ReportScriptError("not song")
						self:stoptweening():linear(0.1):diffusealpha(0)
						return
					end
					self:stoptweening():linear(0.1):diffusealpha(1)
				end
			},

			--[[
			LoadModule("UI/UI.GenerateUIWithButtonAction.lua")(
				{
					Width = 64,
					Height = 42,
					Border = 2,
					Pos = { -PlayerAreaWidth*.5 + 40 , SCREEN_CENTER_Y + 280 },
					Action = function( self )
						local af = self:GetParent():GetParent()
						local state = af:GetChild("NoteField"):GetVisible()
						af:playcommand("ToggleNotefield",{Player=player,State=not state})
						PrefsManager:Set("ShowNotefieldPreview",not state)
					end,
					AddActors = Def.ActorFrame{
						Def.Sprite{
							Texture=THEME:GetPathG("PlayMode","Normal"),
							InitCommand=function(self)
								self:zoom(0.2)
							end
						}
					}
				}
			)..{
				Name="ToggleNotefieldPreview"
			},
			]]

			LoadModule("Wheel/Objects/OptionMenuToggle.lua"){
				Player = player,
				Width = PlayerAreaWidth,
				Touch = CanUseTouchControls,
				ActionPress = function()
					if holdingSelect then holdingSelect = false return end
					if not PlayerPage[player].MenuOpen then
						PlayerPage[player].MenuOpen = not PlayerPage[player].MenuOpen
						mainActorFrame:playcommand("PlayerOpenMenu",{Player=player})
					end
				end
			}..{
				Name="OptionMenuToggle",
				PlayerOpenMenuCommand=function(self,params)
					if params.Player ~= player then return end
					if PlayerPage[player].MenuOpen and PlayerPage[player].Index == 2 then
						self:stoptweening():easeoutexpo(0.2):diffusealpha(0)
					end
					if self:GetChild("ButtonAction") then
						self:GetChild("ButtonAction"):diffusealpha(0)
					end
				end,
				CloseMenuCommand=function(self,params)
					if params.Player ~= player then return end
					if self:GetChild("ButtonAction") then
						self:GetChild("ButtonAction"):diffusealpha(1)
					end
				end
			},
		}

		if NETMAN and NETMAN:IsConnectionEstablished() then
			playerInfo[#playerInfo+1] = LoadModule("Wheel/Objects/OnlineLeaderboard.lua"){
				Player = player,
				Width = PlayerAreaWidth
			}
		end

		PlayerInfoPanes[#PlayerInfoPanes+1] = playerInfo
	end

	local t = Def.ActorFrame{
		OnCommand=function(self)
		end
	}
	-- Here we return the actual Music Wheel Actor.
	t[#t+1] = Def.ActorFrame{
		OnCommand=function(self)
			if not GAMESTATE:IsEventMode() and GAMESTATE:GetNumStagesLeft(GAMESTATE:GetMasterPlayerNumber()) <= 0 then
				lua.ReportScriptError("Woah, there's no more stages, how are you here?")
			end

			SOUND:PlayAnnouncer("select music intro")

			mainActorFrame = self
			-- We use a Input function from the Scripts folder.
			-- It uses a Command function. So you can define all the Commands,
			-- Like MenuLeft is MenuLeftCommand.
			SCREENMAN:GetTopScreen():AddInputCallback(TF_WHEEL.Input(self))
			
			for i = 2,#GroupsAndSongs[CurSong] do
				for a,pn in ipairs( GAMESTATE:GetHumanPlayers() ) do
					if GAMESTATE:GetCurrentSteps(pn) then
						if GroupsAndSongs[CurSong][i] == GAMESTATE:GetCurrentSteps(pn) then
							DiffPos[pn] = i
							LastDiff[pn] = GAMESTATE:GetCurrentSteps(pn):GetDifficulty()
						end
					end
				end
			end

			UpdateSortingFolders()
			

			-- NOTE: This command gets saved to the player's profile data. So it has to be reset
			-- manually once the player has achieved their goal.
			--PROFILEMAN:GetProfile(PLAYER_1):SetGoalType("GoalType_None")
			local StageIndex = GAMESTATE:GetCurrentStageIndex()
			LoadModule("Tools.Discord.lua"):CustomScreenState("Selecting Song (Stage ".. StageIndex+1 .. ")	","",1)
			
			MoveSelection(self,0,GroupsAndSongs)
			
			-- Sleep for 0.2 sec, And then load the current song music.
			if type(GroupsAndSongs[CurSong]) ~= "string" and GroupsAndSongs[CurSong][1]:GetMusicPath() then
				self:GetChild("SongInfoBox"):playcommand("UpdateSongInfo",{Data=GroupsAndSongs[CurSong],visSearch=visSearch})
				for i,player in ipairs( PlayerNumber ) do
					MESSAGEMAN:Broadcast("PlayerSwitchedStep",{ Player = player, Force = true, Index = DiffPos[player]+1, Song = GroupsAndSongs[CurSong] })
				end
			end
			self:sleep(0.2):queuecommand("PlayCurrentSong")
		end,
		
		-- Play Music at start of screen,.
		PlayCurrentSongCommand=function(self)
			if type(newsong) ~= "string" and newsong[1]:GetMusicPath() then
				-- If there's already an audio preview playing on the sound manager, don't perform the change.
				-- newsong[1]:PlayPreviewMusic()
				-- TODO: Make support for the different SampleMode types.

				-- ???: Needed here so it doesn't
				-- freeze when going to player options.
				for i,player in ipairs( PlayerNumber ) do
					local prof = PROFILEMAN:GetProfile(player)
					if newsong[1]:GetGroupName() == prof:GetDisplayName() then
						GAMESTATE:prepare_song_for_gameplay()
						break
					end
				end

				-- Before playing, does the song actually have a song path to play from?
				if string.find(newsong[1]:GetMusicPath(), "/EMPTY") then return end

				SOUND:PlayMusicPart(newsong[1]:GetMusicPath(),newsong[1]:GetSampleStart(),-1,0,0,true,true,false,newsong[1]:GetTimingData())
				-- newsong[1]:PlayPreviewMusic()
			end
		end,

		BeginSearchMessageCommand=function(self)
			BeginSearch()
		end,
		
		-- Do stuff when a user presses left on Pad or Menu buttons.
		-- MenuLeftCommand=function(self) MoveSelection(self,-1,GroupsAndSongs) end,
		MouseWheelUpMessageCommand=function(self,params)
			if AnyPlayerIsOnOptions() or enteredSearch then return end
			MoveSelection(self,-1,GroupsAndSongs)
		end,
		
		-- Do stuff when a user presses Right on Pad or Menu buttons.
		-- MenuRightCommand=function(self) MoveSelection(self,1,GroupsAndSongs) end,
		MouseWheelDownMessageCommand=function(self,params)
			if AnyPlayerIsOnOptions() or enteredSearch then return end
			MoveSelection(self,1,GroupsAndSongs)
		end,
		
		-- Do stuff when a user presses the Down on Pad or Menu buttons.
		MenuDownCommand=function(self) MoveDifficulty(self.pn,1) end,
		
		-- Do stuff when a user presses the Down on Pad or Menu buttons.
		MenuUpCommand=function(self) MoveDifficulty(self.pn,-1) end,
		
		-- Do stuff when a user presses the Back on Pad or Menu buttons.
		BackCommand=function(self) 
			-- Check if User is joined.
			if Joined[self.pn] then
				if PlayerPage[self.pn].MenuOpen then
					PlayerPage[self.pn].OptionListHandler:Back(self.pn)
					return
				end
				if Joined[PLAYER_1] and Joined[PLAYER_2] then
					-- Before unjoining the player, make sure to unload the data
					-- from the notefield for the corresponding player, to avoid timing
					-- data problems.
					self:GetChild("PlayerInfoPanes"):GetChild(self.pn):GetChild("Unload")
					-- If both players are joined, We want to unjoin the player that pressed back.
					Joined[self.pn] = false
					GAMESTATE:UnjoinPlayer(self.pn)
					GAMESTATE:SetCurrentStyle("single")

					MoveSelection(self,0,GroupsAndSongs)
				else
					-- Go to the previous screen.
					CurSearch = ""
					SCREENMAN:GetTopScreen():Cancel()
				end
			end
		end,

		CodeMessageCommand=function(self,param)
			if param.Name == "SortChange" then
				-- Update the sorting mode.
				holdingSelect = true
				performedCode = true
				toggleSortingFromType( "Song" )
				RestoreWheelFromSearch()
				UpdateSortingFolders()
				hasHitAnotherSong = true
				MoveSelection(mainActorFrame,0,GroupsAndSongs,true)
				return
			end

			local function ApplyNewGroupsAndSongs()
				GAMESTATE:Env()["containerStringSongSet"] = nil
				GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,CurGroup,CurSearch,getSortingModeFromType("Group"),getSortingModeFromType("Song"))
				GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]
			end

			-- holdingSelect = param.Name == "PrevStyle" or param.Name == "NextStyle"
			if param.Name == "PrevStyle" or param.Name == "NextStyle" then
				holdingSelect = true
				local offset = ( param.Name == "PrevStyle" and -1 or 1 )
				local oldStyle = m_styleindex
				FetchNextAvailableStyle( Style, offset )
				local oldSongIndex = CurSong
				local cursong = GroupsAndSongs[CurSong][1]

				Songs = ModuleUtils.SongLoader(Style[m_styleindex])
				ApplyNewGroupsAndSongs()

				if (not FindSongInGroup( cursong )) then
					-- So we didn't find the current song, go to the closest available.
					if (inSearchMode or not FindNextAvailableSongInGroup()) then
						CurSong = oldSongIndex
						m_styleindex = oldStyle
						Songs = ModuleUtils.SongLoader(Style[m_styleindex])
						ApplyNewGroupsAndSongs()
						return
					end
				end

				newsong = GetSelectedSong(GroupsAndSongs)
				lastSong = newsong

				hasHitAnotherSong = true
				-- self:GetChild("ModeButtons"):playcommand( (offset == 1 and "Next" or "Prev") .. "Mode")
				MoveSelection(mainActorFrame,0,GroupsAndSongs,true)
				return
			end
			
			if param.Name == "Search" then
				if holdingSelect then holdingSelect = false return end
				-- BeginSearch()
				if not PlayerPage[param.PlayerNumber].MenuOpen then
					PlayerPage[param.PlayerNumber].MenuOpen = not PlayerPage[param.PlayerNumber].MenuOpen
					self:playcommand("PlayerOpenMenu",{Player=param.PlayerNumber})
				else
					self:playcommand("CloseMenu",{Player=param.PlayerNumber})
				end
			end

			if param.Name == "CloseGroup" then
				-- Don't allow this while searching.
				if inSearchMode or inPortal then return end
				-- Neither if the player hasn't joined.
				if not Joined[param.PlayerNumber] then return end
				-- Nor if they're on their menu.
				if PlayerPage[param.PlayerNumber].MenuOpen then return end
				
				-- Set CurSong to the right group.
				for i,v in ipairs(GroupsAndSongs) do
					if v == CurGroup then
						CurSong = i
					end
				end

				self.pn = param.PlayerNumber

				CurGroup = ""
				newsong = ""
				GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,"")
				GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]
				MoveSelection(self,0,GroupsAndSongs,true)

				-- Reset the groups location so we dont bug.
				GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,CurGroup,nil,getSortingModeFromType("Group"),getSortingModeFromType("Song"))
				GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]
				hasHitAnotherSong = true
				MoveSelection(self,0,GroupsAndSongs,true)
			end

			if param.Name == "NextSong" or param.Name == "PrevSong" then
				if not Joined[param.PlayerNumber] then return end
				
				local offset = ( param.Name == "PrevSong" and -1 or 1 )
				if PlayerPage[param.PlayerNumber].MenuOpen then
					if ThreeButtonComp then
						if PlayerPage[param.PlayerNumber].OptionListHandler:InSpecialMenu() then
							PlayerPage[param.PlayerNumber].OptionListHandler:ChangeValue(offset, param.PlayerNumber)
						else
							PlayerPage[param.PlayerNumber].OptionListHandler:MoveSelection(offset,param.PlayerNumber)
						end
					else
						PlayerPage[param.PlayerNumber].OptionListHandler:ChangeValue(offset, param.PlayerNumber)
					end
					return
				end

				MoveSelection(self,offset,GroupsAndSongs)
			end
		end,

		ShowNotEnoughStagesCommand=function(self)
			-- If we're not in event mode, does the player have enough stages to play this song?
			self:GetChild("Wheel"):GetChild("Container"..XOffset):playcommand("ShakeContainer")
			self:GetChild("ItemLocked"):play()
		end,

		PlayerJoinedMessageCommand=function(self,params)
			local newwidth = clamp( scale(aspectRatio,1.60,1.77,300,360), 300, 700 )
			WheelItemWidth = newwidth
			self:GetChild("PlayerInfoPanes"):GetChild(params.Player):visible(true)
			WheelItemsActorFrame:playcommand("UpdateSize",{Width = newwidth - 70})
			self:GetChild("Wheel"):GetWrapperState(1):stoptweening():easeoutexpo(0.2):x( 0 )
			self:GetChild("Wheel"):playcommand("UpdateSize")

			self:GetChild("DragArea"):x( mainActorFrame:GetChild("Wheel"):GetDestX() )
			self:GetChild("DragArea"):GetChild("Detection"):zoomtowidth(WheelItemWidth + 126)

			self:GetChild("SongInfoBox"):playcommand("UpdateSize",{Width = newwidth - 40}):stoptweening():easeoutexpo(0.2)
			:x( SCREEN_CENTER_X )
			GAMESTATE:SetCurrentSteps(params.Player, newsong[DiffPos[params.Player]+1])

			MESSAGEMAN:Broadcast("NotefieldPlayerJoin",{ pn = self.pn, Steps = FetchStepsForWheelItem(self.pn), Style = StyleTEMP[m_styleindex] })
			MESSAGEMAN:Broadcast("PlayerSwitchedStep",{ Player = params.Player, Index = DiffPos[params.Player]+1, Song = newsong })

			-- Turn off the group display as it clips into player 2.
			self:GetChild("Toolkit"):playcommand("Compact")

			self:GetChild("SongInfoBox"):playcommand("UpdateSongInfo",{Data=GroupsAndSongs[CurSong]})
		end,

		PlayerUnjoinedMessageCommand=function(self,params)
			WheelItemWidth = (SCREEN_WIDTH*.8 - PlayerAreaWidth)
			WheelItemsActorFrame:playcommand("UpdateSize",{Width = WheelItemWidth - 70})

			self:GetChild("PlayerInfoPanes"):GetChild(params.Player):visible(false)

			local newMaster = GAMESTATE:GetMasterPlayerNumber()

			for i,v in ipairs(Joined) do
				if Joined[i] then
					MESSAGEMAN:Broadcast("UpdateNotefield",{ pn = i, Steps = FetchStepsForWheelItem(i), Style = StyleTEMP[m_styleindex] })
				end
			end

			local margin = scale( aspectRatio, 1.60, 1.77, .155, .15 )

			self:GetChild("SongInfoBox"):playcommand("UpdateSize",{Width = WheelItemWidth})
			:x( SCREEN_CENTER_X + ((SCREEN_WIDTH * margin) * (newMaster == PLAYER_1 and 1 or -1))  )
			
			self:GetChild("Wheel"):GetWrapperState(1):stoptweening():easeoutexpo(0.2)
			:x( ((SCREEN_WIDTH * margin) * (newMaster == PLAYER_1 and 1 or -1)) )
			self:GetChild("Wheel"):playcommand("UpdateSize")


			self:GetChild("Toolkit"):playcommand("Extend")


			self:GetChild("DragArea"):x(
				mainActorFrame:GetChild("Wheel"):GetX() + 
				mainActorFrame:GetChild("Wheel"):GetWrapperState(1):GetDestX()
			)
			self:GetChild("DragArea"):GetChild("Detection"):zoomtowidth(WheelItemWidth + 200)

			self:GetChild("SongInfoBox"):playcommand("UpdateSongInfo",{Data=GroupsAndSongs[CurSong]})
		end,

		StartSongButtonMessageCommand=function(self)
			for pn in ivalues(PlayerNumber) do
				PlayerPage[pn].MenuOpen = false
				PlayerPage[pn].OptionListHandler:LockInput(true)
				-- if PlayerPage[pn].MenuOpen then
			end
			self:playcommand("Start")
		end,
		
		-- Do stuff when a user presses the Start on Pad or Menu buttons.
		StartCommand=function(self)
			if performedCode then
				performedCode = false
				return
			end

			-- if no player has been assigned, then consider the master player.
			if not self.pn then
				self.pn = GAMESTATE:GetMasterPlayerNumber()
			end

			-- Check if we want to go to ScreenPlayerOptions instead of ScreenGameplay.
			-- Check if player is joined.
			if Joined[self.pn] then 
				if PlayerPage[self.pn].MenuOpen then
					PlayerPage[self.pn].OptionListHandler:ConfirmSelection( self.pn )
					return
				end
			
				-- Check if we are on a group.
				if type(GroupsAndSongs[CurSong]) == "string" then
				
					-- Check if we are on the same group thats currently open,
					-- If not we set the curent group to our new selection.
					if CurGroup ~= GroupsAndSongs[CurSong] then			
						CurGroup = GroupsAndSongs[CurSong]
					-- Same group, Close it.
					else
						CurGroup = ""
					end

					-- SPECIAL: Did the user select the RANDOM option?
					if GroupsAndSongs[CurSong] == "-RANDOM-" then
						-- Show all songs, and pick one at random.
						GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,CurGroup,"",getSortingModeFromType("Group"),getSortingModeFromType("Song"))
						GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]
						-- Force group to be empty
						CurGroup = ""
						newsong = GetPreferredSelectionForRandomOrPortal( GroupsAndSongs )
						RestoreWheelFromSearch()
						hasHitAnotherSong = true
						MoveSelection(mainActorFrame,0,GroupsAndSongs,true)
						return
					end

					-- SPECIAL: Is the user on Portal? They already might have a song highlited from it.
					if GroupsAndSongs[CurSong] == "-PORTAL-" then
						if not CheckStageAvailability(newsong[1]) then
							self:playcommand("ShowNotEnoughStages")
							return
						end
						RestoreWheelFromSearch()
						inPortal = nil
						-- FIX ME: Removing this causes the game to crash if picking a portal song
						-- without having chosen a standaard song previously.
						MoveSelection(mainActorFrame,0,GroupsAndSongs,true)
						self:playcommand("Start")
						return
					end
					
					-- Reset the groups location so we dont bug.
					GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,"")
					GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]
					MoveSelection(self,0,GroupsAndSongs)
					
					-- Set CurSong to the right group.
					if getSortingModeFromType("Group") == "group" then
						for i,v in ipairs(GroupsAndSongs) do
							if v == CurGroup then
								CurSong = i
							end
						end
					end

					-- Set the current group.
					GAMESTATE:Env()["GroupsAndSongs"] = ModuleUtils.GroupSet(Songs,CurGroup,nil,getSortingModeFromType("Group"),getSortingModeFromType("Song"))
					GroupsAndSongs = GAMESTATE:Env()["GroupsAndSongs"]

					if getSortingModeFromType("Group") ~= "group" then
						for i,v in ipairs(GroupsAndSongs) do
							if v == CurGroup then
								CurSong = i
							end
						end
					end

					MoveSelection(self,0,GroupsAndSongs)

					-- FetchOnlineInfoCharts(GroupsAndSongs)
					
				-- Not on a group, Start song.
				else
					if not CheckStageAvailability(newsong[1]) then
						self:playcommand("ShowNotEnoughStages")
						return
					end
					
					--We use PlayMode_Regular for now.
					GAMESTATE:SetCurrentPlayMode("PlayMode_Regular")
					
					--Set the song we want to play.
					GAMESTATE:SetCurrentSong(newsong[1])
					GAMESTATE:SetPreferredSong( newsong[1] )
					
					-- Check if 2 players are joined.
					if Joined[PLAYER_1] and Joined[PLAYER_2] then
				
						-- Is someone on a menu? Stop!
						if PlayerPage[PLAYER_1].MenuOpen or PlayerPage[PLAYER_2].MenuOpen then
							return
						end
						-- If they are, We will use Versus.
						GAMESTATE:SetCurrentStyle('versus')
					
						-- Save Profiles.
						-- PROFILEMAN:SaveProfile(PLAYER_1)
						-- PROFILEMAN:SaveProfile(PLAYER_2)
					
						-- Set the Current Steps to use.
						GAMESTATE:SetCurrentSteps(PLAYER_1,newsong[DiffPos[PLAYER_1]+1])
						GAMESTATE:SetCurrentSteps(PLAYER_2,newsong[DiffPos[PLAYER_2]+1])
					else
				
						-- If we are single player, Use Single.
						GAMESTATE:SetCurrentStyle(TF_WHEEL.StyleDB[Style[m_styleindex]])
					
						-- Save Profile.
						-- PROFILEMAN:SaveProfile(self.pn)
					
						-- Set the Current Step to use.
						GAMESTATE:SetCurrentSteps(self.pn,newsong[DiffPos[self.pn]+1])
					end

					-- If the song comes from an external source like a USB, make
					-- sure to load it.
					local checks = GAMESTATE:prepare_song_for_gameplay()

					if checks ~= "success" then
						lua.ReportScriptError("Debug: "..checks)
						return
					end

					local isSafe,reason = GAMESTATE:CanSafelyEnterGameplay()

					if not isSafe then
						lua.ReportScriptError("Debug: "..reason)
						return
					end
					
					-- Store wheel when finished and still in search mode.
					if inSearchMode then
						if not KeepSearch then
							CurSearch = nil
						end
						-- RestoreWheelFromSearch()
					end

					doneWithInput = true

					self:stoptweening():queuecommand("StartSong")
				end
			else
				-- Add to joined list.
				Joined[self.pn] = true

				-- if THEME:GetMetric("GameState","AllowLateJoin") == false then return end
				-- If no player is active Join.
				GAMESTATE:JoinPlayer(self.pn)
				
				-- Load the profles.
				GAMESTATE:LoadProfiles()
				
				
				MoveSelection(self,0,GroupsAndSongs)
			end			
		end,

		PlayerOpenMenuCommand=function(self,params)
			local isEveryoneOnPause = true
			for i,pn in ipairs(GAMESTATE:GetHumanPlayers()) do
				if not PlayerPage[pn].MenuOpen then
					isEveryoneOnPause = false
				end
			end
			if not isEveryoneOnPause then return end

			self:playcommand("DimMainItems")
		end,

		DimMainItemsCommand=function(self)
			self:GetChild("Wheel"):stoptweening():linear(0.1):diffuse( color("#777777") )
			self:GetChild("SongInfoBox"):stoptweening():linear(0.1):diffuse( color("#777777") )
		end,

		RestoreItemDiffuseCommand=function(self)
			self:GetChild("Wheel"):stoptweening():linear(0.1):diffuse( Color.White )
			self:GetChild("SongInfoBox"):finishtweening():linear(0.1):diffuse( Color.White )
		end,
		
		-- Change to ScreenGameplay.
		StartSongCommand=function(self)

			-- TODO: Add check for different music comment modes.
			SOUND:PlayAnnouncer("select music comment general")

			local screentogo = UseTwoStepDiff and "ScreenPlayerOptionsLua" or "ScreenGameplay"
			SCREENMAN:PlayStartSound()
			if not UseTwoStepDiff then
				SOUND:Volume(0,0.3)
			end
			SCREENMAN:GetTopScreen():SetNextScreenName(screentogo):StartTransitioningScreen("SM_GoToNextScreen")
		end,

		PlayerInfoPanes,

		--[[
		ModeButtons..{
			OnCommand=function(self)
				self:xy( 0,0 )
				self:finishtweening():diffusealpha(0):zoom(0.3):easeoutexpo(0.2):diffusealpha(1):zoom(0.4)
			end,
			OffCommand=function(self)
				self:stoptweening():easeinexpo(0.2):diffusealpha(0):zoom(0.3)
			end,
		},
		]]

		WIAF,
		
		-- Load the wheel.
		Wheel..{
			OnCommand=function(self)
				local margin = scale( aspectRatio, 1.60, 1.77, .155, .15 )
				if needsSDVersion then
					margin = .165
				end
				local side = GAMESTATE:GetMasterPlayerNumber() == PLAYER_1 and 1 or -1
				local xWheelPos = GAMESTATE:GetNumPlayersEnabled() > 1 and 0 or ((SCREEN_WIDTH * margin) * side)
				self:xy( SCREEN_CENTER_X,  SCREEN_CENTER_Y )
				:zoom( 1.3 )
				self:AddWrapperState()
				self:GetWrapperState(1):x( xWheelPos )

				self.GetTotalXPosition = function()
					return self:GetX() + self:GetWrapperState(1):GetX()
				end

				self:addx(20):diffusealpha(0):easeoutexpo(0.5):diffusealpha(1):addx(-20)
			end,
			OffCommand=function(self)
				self:stoptweening():easeinexpo(0.2):diffusealpha(0):addx(20)
			end,
		},
		
		Def.ActorFrame{
			Name="Toolkit",
			InitCommand=function(self)
				self.SetPosition = SCREEN_CENTER_X
				self:y( 28 )
				if needsSDVersion then
					self.SetPosition = SCREEN_CENTER_X + 100
				end
				-- Player 2 is going to need a special offset.
				if getJoinedPlayers() == 1 and GAMESTATE:GetMasterPlayerNumber() == PLAYER_2 then
					self.SetPosition = 280
				end
				self:x( self.SetPosition )
			end,
			OnCommand=function(self)
				self:playcommand("SearchPerformed",{ String = CurSearch })
				-- self:GetChildAt(3):x(CurSearch ~= "" and -75 or -200 )
			end,
			CompactCommand=function(self)
				self:GetChild("GroupChooser"):stoptweening():linear(0.1):diffusealpha(0)
				self:GetChild("SortingMethodsInsideFolder"):x(74 + 30)
				self:GetChild("SortingMethodsFolders"):x(-74 + 30)
				self:GetChild("SearchBar"):x( -200 + 30 )
				self:stoptweening():easeoutexpo(0.3):x( SCREEN_CENTER_X ) 
			end,
			ExtendCommand=function(self)
				self.SetPosition =GAMESTATE:GetMasterPlayerNumber() == PLAYER_1 and SCREEN_CENTER_X or 280
				self:GetChild("GroupChooser"):stoptweening():linear(0.1):diffusealpha(1)
				self:GetChild("SortingMethodsInsideFolder"):x(74)
				self:GetChild("SortingMethodsFolders"):x(-74)
				self:GetChild("SearchBar"):x(-200)
				self:stoptweening():easeoutexpo(0.3):x( self.SetPosition ) 
			end,
			LoadModule("Wheel/Objects/OutsideFolderSort.lua"){
				Width = (300/3) + 46,
				Action = function(self)
					-- Update the sorting mode.
					toggleSortingFromType( "Group" )
					CurGroup = GetGroupNameBasedOnSort(newsong[1])
					RestoreWheelFromSearch()
					UpdateSortingFolders()
					hasHitAnotherSong = true
					MoveSelection(mainActorFrame,0,GroupsAndSongs,true)
				end
			}..{ InitCommand=function(self) self:x( -74 ) end },
			LoadModule("Wheel/Objects/InsideFolderSort.lua"){
				Width = (300/3) + 46,
				Action = function(self)
					-- Update the sorting mode.
					toggleSortingFromType( "Song" )
					RestoreWheelFromSearch()
					UpdateSortingFolders()
					hasHitAnotherSong = true
					MoveSelection(mainActorFrame,0,GroupsAndSongs,true)
				end
			}..{ InitCommand=function(self) self:x( 74 ) end },
			LoadModule("Wheel/Objects/Searchbar.lua"){
				Width = 300/3,
				Search = BeginSearch
			},
			LoadModule("Wheel/Objects/GroupChooser.lua"){
				Width = needsSDVersion and 220 or 400,
			}..{ InitCommand=function(self) self:x( needsSDVersion and 260 or 350 ) end }
		},

		LoadModule("Wheel/Objects/SongInfoBox.lua"){
			Width = WheelItemWidth+4 - ( getJoinedPlayers() > 1 and 40 or 0),
			MoveFunction = function( pos )
				MoveSelection( mainActorFrame, pos, GroupsAndSongs )
			end,
			Touch = CanUseTouchControls,
			StartFunction = function()
				if mouseIsInDragMode then
					return
				end
				mainActorFrame:playcommand("Start")
			end
		}..{
			OnCommand=function(self)
				local wheel = self:GetParent():GetChild("Wheel")
				self:zoom(1.15):x( wheel.GetTotalXPosition() - 20 )
				:diffusealpha(0):easeoutquint(0.4):diffusealpha(1)
			end,
			OffCommand=function(self)
				self:easeinexpo(0.2):diffusealpha(0)
			end
		},
		
		--[[
		Def.Quad{
			Name="Slider",
			OnCommand=function(self)
				local wheel = self:GetParent():GetChild("Wheel")
				self:zoomto(8,32):diffuse(1,0,0,1)
				:xy( (wheel:GetTotalXPosition() + WheelItemWidth*.6) - 4,-180+(350*(CurSong/#GroupsAndSongs)))
				:addx(20):easeoutexpo(0.5):addx(-20)
			end,
			UpdateSizeCommand=function(self)
				self:x(wheel.GetTotalXPosition() + WheelItemWidth*.5)
			end,
			OffCommand=function(self)
				self:stoptweening():easeinquint(0.2):addx(20):diffusealpha(0)
			end
		},
		]]

		Def.Sound{ Name="ChangeSound", File=THEME:GetPathS("","click") },
		Def.Sound{ Name="ItemLocked", File=THEME:GetPathS("MusicWheel","locked") },

		-- TEST: Attempt to make a swiping system for touch screens.
		-- suggestions on how to improve this are welcome.
		LoadModule("UI/UI.HoldClickArea.lua"){
			Width = WheelItemWidth + ( getJoinedPlayers() == 1 and 180 or 120),
			Height = SCREEN_HEIGHT,
			ReturnAdjacentActorFrame = true,
			Position = function(self)
				local margin = scale( aspectRatio, 1.60, 1.77, .155, .15 )
				local side = GAMESTATE:GetMasterPlayerNumber() == PLAYER_1 and 1 or -1
				local xWheelPos = GAMESTATE:GetNumPlayersEnabled() > 1 and SCREEN_CENTER_X or (SCREEN_CENTER_X + ((SCREEN_WIDTH * margin) * side))
				self:xy( xWheelPos,  SCREEN_CENTER_Y )
			end,
			Action = function(self)
				-- test to track mouse position.
				local MouseX,MouseY = INPUTFILTER:GetMouseX(), INPUTFILTER:GetMouseY()
				-- Is the mouse inside of the area where the wheel is?
				if not mouseIsInArea then
					lastYPosMouse = MouseY
					TEMPYPosMouse = MouseY
					mouseIsInArea = true
					mouseIsInDragMode = false
					hasHitAnotherSong = false -- Used to mark the audio playback when lifting the finger on a different song.
				end
				
				-- This represent the distance between each item to count as a scroll.
				local margin = 42
				local distance = (lastYPosMouse-MouseY)
				
				-- Relative distance to move the cursor to.
				offsettosum = TEMPYPosMouse-MouseY
				self:GetChild("Wheel"):addy( -offsettosum * self:GetChild("Wheel"):GetZoom() )

				if math.abs(offsettosum) > 0 then
					mouseIsInDragMode = true
				end

				-- Set the position from that relative distance to the new position we happen to have.
				TEMPYPosMouse = MouseY

				
				-- Cursor is on the next or previous song, switch to it.
				if math.abs(distance) > margin then
					local posMove = (distance > margin) and 1 or -1
					self:GetChild("Wheel"):finishtweening():linear(0):y( SCREEN_CENTER_Y )
					MoveSelection( self, posMove, GroupsAndSongs, true )
					hasHitAnotherSong = true
					offsettosum = 0
					lastYPosMouse = MouseY
				end
			end,
			ActionUnclick = function(self)
				if not mouseIsInArea then return end
				mouseIsInArea = false
				mouseIsInDragMode = false
				self:GetChild("Wheel"):finishtweening():decelerate(0.2):y( SCREEN_CENTER_Y )
				MoveSelection( self, 0, GroupsAndSongs, true )
			end
		}..{ Name="DragArea" }
	}

	return t
end
